#pragma once
#include <avr/io.h>
#include "I2C.h"

void TWIinitMaster(char TWI_twbr){
	TWBR = TWI_twbr;										//value for transmission frequency setting
	TWCR = 1 << TWEN;										// enable I2C
}

char TWIsendStart(char TWIslaveAddress){
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTA);				//generate start
	while(!(TWCR & (1<<TWINT)));							//wait for flag of finished task
	if ((TWSR & 0xF8) != 0x08){								//if start hasn't been generated
		return 1;
	}
	TWDR = TWIslaveAddress << 1;
	TWCR = (1<<TWINT) | (1<<TWEN);							//send address
	while(!(TWCR & (1<<TWINT)));							//wait for flag of finished task
	if ((TWSR & 0xF8) != 0x18){								//if address and write bit hasn't been transmitted + ACK bit received
		TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);			//generate stop
		return 1;
	}
	return 0;
}

char TWIsendByte(char TWIdata){
	TWDR = TWIdata;
	TWCR = (1<<TWINT) | (1<<TWEN);							//send byte
	while(!(TWCR & (1<<TWINT)));							//wait for flag of finished task
	if ((TWSR & 0xF8) != 0x28){								//if byte hasn't been transmitted + ACK bit received
		TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);			//generate stop
		return 1;
	}
	return 0;
}

void TWIstop(){
	TWCR = ((1<<TWINT) | (1<<TWEN)) | (1<<TWSTO);			//generate stop
}

char TWIreceive(char TWIaddress, char *TWIdataArray, unsigned int TWInumberOfBytes){
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTA);				//generate start
	while(!(TWCR & (1<<TWINT)));							//wait for flag of finished task
	if ((TWSR & 0xF8) != 0x08){								//if start hasn't been generated
		return 1;
	}
	TWDR = (TWIaddress<<1) | 1;								//prepare address and set read bit
	TWCR = (1<<TWINT) | (1<<TWEN);							//send address and R/W bit
	while(!(TWCR & (1<<TWINT)));							//wait for flag of finished task
	if ((TWSR & 0xF8) != 0x40){								//if address and read bit wasn't transmitted + ACK bit received
		TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);			//generate stop
		return 1;
	}
	unsigned int TWIreceiveCount = 0;						//counter of received bytes
	while(TWIreceiveCount < (TWInumberOfBytes - 1)){		//while next received byte won't be the last byte
		TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWEA);			//receive byte and return ACK
		while(!(TWCR & (1<<TWINT)));						//wait for flag of finished task
		if ((TWSR & 0xF8) != 0x50){							//if byte wasn't received and ACK returned
			TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);		//generate stop
			return 1;
		}
		TWIdataArray[TWIreceiveCount] = TWDR;				//read received byte
		TWIreceiveCount++;									//increment counter of received bytes
	}
	TWCR = (1<<TWINT) | (1<<TWEN);							//receive the last byte and return NACK
	while(!(TWCR & (1<<TWINT)));							//wait for flag of finished task
	if ((TWSR & 0xF8) != 0x58){								//if byte wasn't received and NACK returned
		TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);			//generate stop
		return 1;
	}
	TWIdataArray[TWIreceiveCount] = TWDR;					//read the last byte
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);				//generate stop
	return 0;
}