#include <avr/io.h>
#include <util/delay.h>
#include "LCD_2porty.h"

void LCDsendCommand(char LCDcommand){
	LCD_PORT_DATA &= 0xF0 | (LCDcommand>>4);					//set all logic nulls of upper half of command data
	LCD_PORT_COMMAND |= 1<<LCD_E;
	LCD_PORT_DATA |= LCDcommand>>4;								//set all logic ones of upper half of command data
	_delay_us(1);
	LCD_PORT_COMMAND &= ~(1<<LCD_E);							//send upper half of data
	_delay_us(1);
	LCD_PORT_DATA &= 0xF0 | (LCDcommand & 0x0F);				//set all logic nulls of lower half of command data
	LCD_PORT_COMMAND |= 1<<LCD_E;
	LCD_PORT_DATA |= LCDcommand & 0x0F;							//set all logic ones of lower half of command data
	_delay_us(1);
	LCD_PORT_COMMAND &= ~(1<<LCD_E);							//send lower half of data
	_delay_ms(2);
}

void LCDwrite(char LCDchar){
	LCD_PORT_DATA &= 0xF0 | (LCDchar>>4);						//set all logic nulls of upper half of character
	LCD_PORT_DATA |= LCDchar>>4;								//set all logic ones of upper half of character
	LCD_PORT_COMMAND |= (1<<LCD_E) | (1<<LCD_RS);				//set E and RS for sending DDRAM or CGRAM data
	_delay_us(1);
	LCD_PORT_COMMAND &= ~(1<<LCD_E);							//send upper half of data
	_delay_us(1);
	LCD_PORT_DATA &= 0xF0 | (LCDchar & 0x0F);					//set all logic nulls of lower half of character
	LCD_PORT_DATA |= LCDchar & 0x0F;							//set all logic ones of lower half of character
	LCD_PORT_COMMAND |= 1<<LCD_E;								//send lower half of data
	_delay_us(1);
	LCD_PORT_COMMAND &= ~((1<<LCD_E) | (1<<LCD_RS));			//set E and RS low
	_delay_us(50);
}

void LCDinit(){
	LCD_DDR_DATA |= 0x0F;										//set LCD data pins as output
	LCD_DDR_COMMAND |= (1<<LCD_E) | (1<<LCD_RW) | (1<<LCD_RS);	//set LCD command pins as output
	LCD_PORT_COMMAND |= 1<<LCD_E;
	LCD_PORT_DATA |= 0x02;										//send LCD initialization for 4 data wire communication
	_delay_us(1);
	LCD_PORT_COMMAND &= ~(1<<LCD_E);
	_delay_us(40);
	LCDsendCommand(0b00101000);									//function set = 4 bit communication, 2 line display, 5x8 dots font
	LCDsendCommand(0b00001100);									//display on/off control = display on, cursor off, blinking of cursor position character off
	LCDsendCommand(0b00000110);									//entry mode set = cursor move direction right, accompanies display shift off
	LCDsendCommand(LCD_CGRAM_ADDR 0x00);
	
	//character 0
	LCDwrite(0b00011);
	LCDwrite(0b00111);
	LCDwrite(0b01111);
	LCDwrite(0b11111);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	
	//character 1
	LCDwrite(0b11000);
	LCDwrite(0b11100);
	LCDwrite(0b11110);
	LCDwrite(0b11111);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	
	//character 2
	LCDwrite(0b11111);
	LCDwrite(0b01111);
	LCDwrite(0b00111);
	LCDwrite(0b00011);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	
	//character 3
	LCDwrite(0b11111);
	LCDwrite(0b11110);
	LCDwrite(0b11100);
	LCDwrite(0b11000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	
	//character 4
	LCDwrite(0b00011);
	LCDwrite(0b00111);
	LCDwrite(0b01111);
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	
	//character 5
	LCDwrite(0b11000);
	LCDwrite(0b11100);
	LCDwrite(0b11110);
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	
	//character 6
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	LCDwrite(0b11111);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	
	//character 7
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b00000);
	LCDwrite(0b01110);
	LCDwrite(0b10001);
	LCDwrite(0b10001);
	LCDwrite(0b10001);
	LCDwrite(0b01110);
	
	LCDsendCommand(LCD_CLEAR);									//clear display and set cursor position 0
}

void LCDprintString(char *LCDstring){
	char LCDprintStringCount = 0;								//counter of characters in string
	while(LCDstring[LCDprintStringCount]){						//while data at counter position is character (!=0) (it's not the end of string)
		LCDwrite(LCDstring[LCDprintStringCount]);				//print character
		LCDprintStringCount++;									//increase counter
	}
}

void LCDgoto(char LCDgotoCol, char LCDgotoRow){
	char LCDgotoAddress;
	switch(LCDgotoRow){											//calculate the address based on row and column information
		case 1:
		LCDgotoAddress = LCD_1ROW;
		break;
		case 2:
		LCDgotoAddress = LCD_2ROW;
		break;
		case 3:
		LCDgotoAddress = LCD_3ROW;
		break;
		default:
		LCDgotoAddress = LCD_0ROW;
		break;
	}
	LCDgotoAddress += LCDgotoCol;
	LCDsendCommand(0x80 | (LCDgotoAddress & 0x7F));				//send DDRAM address = address of cursor position
}

char LCDread(){
	LCD_PORT_DATA &= 0xF0;										//set data pins to zero (not connected to pull-up resistors internally after set pins as an input)
	LCD_DDR_DATA &= 0xF0;										//set data pins as an input
	LCD_PORT_COMMAND |= 1<<LCD_RW;								//set RW for reading
	_delay_us(1);
	LCD_PORT_COMMAND |= 1<<LCD_E;								//set E to be able to read data
	_delay_us(1);
	char LCDreadedData = (LCD_PIN_DATA & 0x0F)<<4;				//read upper half of data and shift to right position
	LCD_PORT_COMMAND &= ~(1<<LCD_E);
	_delay_us(1);
	LCD_PORT_COMMAND |= 1<<LCD_E;
	_delay_us(1);
	LCDreadedData |= LCD_PIN_DATA & 0x0F;						//read lower half of data
	LCD_PORT_COMMAND &= ~(1<<LCD_RW) | (1<<LCD_E);
	LCD_DDR_DATA |= 0x0F;										//set data pins as an output
	return LCDreadedData;
}

char LCDreadAddress(){
	return LCDread() & 0x7F;									//read data except MSB is DDRAM address = cursor position address
}

char LCDbussy(){
	return LCDread() & 0x80;									//MSB of read data is busy flag of LCD
}

void LCDbigDigit(char LCDbigDigitNum){
	const char fontUp[10][3] = {{4,6,5},       {0,255,16},  {0,6,5}, {0,6,5},    {255,16,255}, {255,6,3}, {4,6,1},   {2,6,255}, {4,6,5},     {4,6,5}};
	const char fontMid[10][3] = {{255,16,255}, {16,255,16}, {4,6,3}, {16,6,255}, {2,6,255},    {6,6,5},   {255,6,5}, {16,4,3},  {255,6,255}, {2,6,255}};
	const char fontDown[10][3] = {{2,6,3},     {6,6,6},     {2,6,3}, {2,6,3},    {16,16,6},    {2,6,3},   {2,6,3},   {16,6,16}, {2,6,3},     {2,6,3}};
	char LCDbigDigitPosition = LCDreadAddress();
	if(LCDbigDigitNum < 10){
		for(char LCDbigDigitCount = 0; LCDbigDigitCount < 3; LCDbigDigitCount++){
			LCDwrite(fontUp[LCDbigDigitNum][LCDbigDigitCount]);
		}
		LCDsendCommand(LCD_GOTO_ADDR (LCDbigDigitPosition + LCD_1ROW));
		for(char LCDbigDigitCount = 0; LCDbigDigitCount < 3; LCDbigDigitCount++){
			LCDwrite(fontMid[LCDbigDigitNum][LCDbigDigitCount]);
		}
		LCDsendCommand(LCD_GOTO_ADDR (LCDbigDigitPosition + LCD_2ROW));
		for(char LCDbigDigitCount = 0; LCDbigDigitCount < 3; LCDbigDigitCount++){
			LCDwrite(fontDown[LCDbigDigitNum][LCDbigDigitCount]);
		}
		LCDsendCommand(LCD_GOTO_ADDR (LCDbigDigitPosition + 3));
	}
	else{
		LCDwrite(7);
		LCDsendCommand(LCD_GOTO_ADDR (LCDbigDigitPosition + LCD_1ROW));
		LCDwrite(7);
		LCDsendCommand(LCD_GOTO_ADDR (LCDbigDigitPosition + 1));
	}
}

void LCDputFloatTwoDecPlaces(float LCDputFloatNumber){
	char LCDputFloatString[6];									//buffer for string
	int LCDputFloatInt = (int)LCDputFloatNumber;				//create integer from float
	LCDputFloatNumber -= LCDputFloatInt;
	LCDputFloatNumber *= 100;
	int LCDputFloatDecimal = (int)LCDputFloatNumber;			//create integer including two decimal places
	if(LCDputFloatDecimal < 0){									//if float was minus remove "-" from decimal part of number
		LCDputFloatDecimal *= -1;
		if(!LCDputFloatInt){									//if float was minus but
			LCDwrite('-');
		}
	}
	itoa(LCDputFloatInt,LCDputFloatString,10);
	LCDprintString(LCDputFloatString);
	LCDwrite('.');
	if(LCDputFloatDecimal < 10){								//if first decimal place is zero, print zero because function itoa doesn't print zero in front of nonzero value
		LCDwrite('0');
	}
	itoa(LCDputFloatDecimal,LCDputFloatString,10);
	LCDprintString(LCDputFloatString);
}

void LCDputFloatOneDecPlace(float LCDputFloatNumber){
	char LCDputFloatString[6];
	int LCDputFloatInt = (int)LCDputFloatNumber;
	LCDputFloatNumber -= LCDputFloatInt;
	LCDputFloatNumber *= 10;
	int LCDputFloatDecimal = (int)LCDputFloatNumber;
	if(LCDputFloatDecimal < 0){
		LCDputFloatDecimal *= -1;
		if(!LCDputFloatInt){
			LCDwrite('-');
		}
	}
	itoa(LCDputFloatInt,LCDputFloatString,10);
	LCDprintString(LCDputFloatString);
	LCDwrite('.');
	itoa(LCDputFloatDecimal,LCDputFloatString,10);
	LCDprintString(LCDputFloatString);
}

void LCDputInt(int LCDputIntNumber){
	char LCDputIntString[6];
	itoa(LCDputIntNumber,LCDputIntString,10);
	LCDprintString(LCDputIntString);
}