#pragma once
/* Pin functions of defined ports:
    LCD_PORT_DATA:
        | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
        | - | - | - | - |D7 |D6 |D5 |D4 |
    LCD_PORT_COMMAND:
        depends on definition of each data lines (RW, RS, E)
*/

#ifndef LCD_PORT_DATA
#	define LCD_PORT_DATA PORTB
#	warning "LCD_PORT_DATA not defined for LCD_2porty.h. Automaticaly defined as PORTB"
#endif

#ifndef LCD_PORT_COMMAND
#	define LCD_PORT_COMMAND PORTD
#	warning "LCD_PORT_COMMAND not defined for LCD_2porty.h. Automaticaly defined as PORTD"
#endif

#ifndef LCD_DDR_DATA
#	define LCD_DDR_DATA DDRB
#	warning "LCD_DDR_DATA not defined for LCD_2porty.h. Automaticaly defined as DDRB"
#endif

#ifndef LCD_DDR_COMMAND
#	define LCD_DDR_COMMAND DDRD
#	warning "LCD_DDR_COMMAND not defined for LCD_2porty.h. Automaticaly defined as DDRD"
#endif

#ifndef LCD_PIN_DATA
#	define LCD_PIN_DATA PINB
#	warning "LCD_PIN_DATA not defined for LCD_2porty.h. Automaticaly defined as PINB"
#endif

#ifndef LCD_RW
#	define LCD_RW 4
#	warning "LCD_RW not defined for LCD_2porty.h. Automaticaly defined as PORTD4"
#endif

#ifndef LCD_RS
#	define LCD_RS 5
#	warning "LCD_RS not defined for LCD_2porty.h. Automaticaly defined as PORTD5"
#endif

#ifndef LCD_E
#	define LCD_E 6
#	warning "LCD_E not defined for LCD_2porty.h. Automaticaly defined as PORTD6"
#endif

#define LCD_0ROW 0x00											//start of 1st line
#define LCD_1ROW 0x40											//start of 2nd line
#define LCD_2ROW 0x14											//start of 3rd line
#define LCD_3ROW 0x54											//start of 4th line

#define LCD_CLEAR 0x01											//use: LCDsendCommand(LCD_clear);
#define LCD_CGRAM_ADDR 0x40 |									//set CGRAM address (adress of custom characters) usage: LCDsendCommand(LCD_CGRAMaddr 0x08);
#define LCD_GOTO_ADDR 0x80 |									//set DDRAM address (address of cursor position) usage: LCDsendCommand(LCD_GOTO_ADDR 0x27);
#define LCD_HOME 0x02										//set cursor address 0, returns display from being shifted to original position, doesn't clear display
#define LCD_shiftDisplayLeft 0x1C							//shifts display to left
#define LCD_shiftDisplayRight 0x18								//shifts display to right

/**
	Send instruction for LCD;
*/
void LCDsendCommand(char LCDcommand);

/**
	Initialize 4bit communication with LCD, 2 line display,
	cursor off, blinking char off, generate custom characters for 3 line numbers, clear display.
*/
void LCDinit();

/**
	Send data byte to LCD. DDRAM or CGRAM data.
*/
void LCDwrite(char LCDchar);

/**
	Print string to cursor position.
*/
void LCDprintString(char *LCDstring);

/**
	Set cursor position row, column.
*/
void LCDgoto(char LCDgotoCol, char LCDgotoRow);

/**
	Return busy flag (7th bit) and cursor position address (7bit) data.
*/
char LCDread();

/**
	Return address of current cursor position.
*/
char LCDreadAddress();

/**
	If busy, return 1. Else return 0.
*/
char LCDbussy();

/**
	Print three line height digit.
	If LCDbigDigitNum is 10, print colon.
*/
void LCDbigDigit(char LCDbigDigitNum);

/**
	Print float with a precision of two decimal places.
*/
void LCDputFloatTwoDecPlaces(float LCDputFloatNumber);

/**
	Print float with a precision of one decimal place.
*/
void LCDputFloatOneDecPlace(float LCDputFloatNumber);

/**
	Print integer.
*/
void LCDputInt(int LCDputIntNumber);