/*
 * meteostanica1.0.c
 *
 * Created: 29. 7. 2019 15:36:21
 * Author : grega
 */ 

/*
	FUSE SETTINGS:
		LOW FUSE:	0xCF
		HIGH FUSE:	0xD9
	
	EEPROM MEMORY USAGE:
		address		usage
		0x00		statusbyte_mask
		0x01		volume
		0x02		hours of alarm
		0x03		minutes of alarm
*/

#define F_CPU 16000000
#define LCD_PORT_DATA PORTB
#define LCD_PORT_COMMAND PORTD
#define LCD_DDR_DATA DDRB
#define LCD_DDR_COMMAND DDRD
#define LCD_PIN_DATA PINB
#define LCD_RW PORTD4
#define LCD_RS PORTD5
#define LCD_E PORTD6
#define LCD_LIGHT PORTD7

#define SHTC3_ADDR 0x70
#define SHTC3_SLEEP_MSB 0xB0
#define SHTC3_SLEEP_LSB 0x98
#define SHTC3_WAKEUP_MSB 0x35
#define SHTC3_WAKEUP_LSB 0x17
#define SHTC3_MEASURE_MSB 0x5C
#define SHTC3_MEASURE_LSB 0x24

#define DS3231_ADDR 0b1101000

#define SHR_PORT PORTC
#define SHR_PIN PINC
#define SHR_CLK PORTC0
#define SHR_DATA PORTC1
#define SHR_LD PORTC2
#define SHR_SWITCHES 1
#define SHR_RECEIVED 2

#define WT_PORT PORTB
#define WT_DATA PORTB4
#define WT_PIN PINB
#define WT_BUSSY PINB5
#define WT588D_WAITING_TIMEOUT 5000
#define WT588D_STOP 0xFE
#define WT_MENU_IN 0x5A
#define WT_MENU_OUT 0x5B
#define WT_MENU_POPIS 0x5C
#define WT_BUDIK_VYPNUTY 0x4F
#define WT_BUDIK_ZAPNUTY 0x4E
#define WT_VO_VNUTRI_JE 0x49
#define WT_VONKU_JE 0x4D
#define WT_NIZKA_BATERIA 0x59
#define WT_JE 0x46
#define WT_SU 0x55
#define WT_1 0x57
#define WT_2 0x56
#define WT_10 0x0A
#define WT_18 0x12
#define WT_20 0x14
#define WT_2000 0x1D
#define WT_HODINA 0x51
#define WT_HODINY 0x50
#define WT_HODIN 0x47
#define WT_MINUTA 0x53
#define WT_MINUTY 0x52
#define WT_MINUT 0x48
#define WT_PONDELOK 0x33
#define WT_UTOROK 0x34
#define WT_STREDA 0x35
#define WT_STVRTOK 0x36
#define WT_PIATOK 0x37
#define WT_SOBOTA 0x38
#define WT_NEDELA 0x39
#define WT_10th 0x27
#define WT_18th 0x2F
#define WT_1st_MINUS_1 0x1D
#define WT_JANUAR_MINUS_1 0x39
#define WT_MINUS 0x58
#define WT_CELA 0x4A
#define WT_STUPNOV 0x4B
#define WT_PERCENT 0x4C
#define WT_NASTAVTE 0x5D
#define WT_CAS 0x5E
#define WT_DATUM 0x5F
#define WT_DEN_V_TYZDNI 0x60
#define WT_POTVRTDE_TLACIDLOM 0x61
#define WT_POTVRDENE 0x62
#define WT_JEDNOTKY 0x63
#define WT_DESIATKY 0x64
#define WT_CHYBNY 0x6D
#define WT_DNA 0x67
#define WT_MESIAC 0x66
#define WT_ROKOV 0x65
#define WT_CAS_BUDIKA 0x6E
#define WT_HLASITOST 0x6A
#define WT_HLASOVY_VYSTUP 0x68
#define WT_ROZPOZNAVANIE_HLASU 0x69
#define WT_ZAPNUTE 0x6B
#define WT_VYPNUTE 0x6C
#define WT_INTRO 0x6F

#define ALARM 7
#define RTC_INT1 6
#define SW_TOP 5
#define SW5 4
#define SW4 3
#define SW3 2
#define SW2 1
#define SW1 0

#define MENU 7

#define MASK_SW5 7
#define MASK_WRONG_DATE 6
#define MASK_LISTENING 5
#define MASK_SPEAKING 4
#define MASK_RH_T_OUT 3
#define MASK_RH_T_IN 2
#define MASK_DATE 1
#define MASK_ALARM 0

#define FLAGS_SETTINGS_SPEAKING_ENABLED 7
#define FLAGS_ALARM_FIRST 6
#define FLAGS_RF_NOT_SAME 5
#define FLAGS_RF_FIRST 4
#define FLAGS_USART 3
#define FLAGS_SAY 2
#define FLAGS_MENU_LCD 1
#define FLAGS_GO_TO_MENU 0

#define RV_WAIT 0x00
#define RV_DELETE_INSTRUCTIONS 0x04
#define RV_IMPORT_INSTRUCTIONS 0x21
#define RV_COMPACT_MODE 0x37
#define USART_READ_TIMEOUT 200;			//time in ms - how long is function USART_Read waiting for data if not received

#define RF_NEW_DATA 0x0C
#define RF_START 0x0A
#define RF_CONTINUE 0x0D
#define RF_STOP 0x0F

#define SENSOR_H_TENS 0
#define SENSOR_H_ONES 1
#define SENSOR_T_TENS 2
#define SENSOR_T_ONES 3
#define SENSOR_T_DECIMAL 4
#define SENSOR_STATUS 5
#define SENSOR_STATUS_MINUS 0
#define SENSOR_STATUS_LOW_BATTERY 1

#define HOURS_TENS 3
#define HOURS_ONES 2
#define MINUTES_TENS 1
#define MINUTES_ONES 0

#define DAY_TENS 0
#define DAY_ONES 1
#define MONTH_TENS 2
#define MONTH_ONES 3
#define YEAR_TENS 4
#define YEAR_ONES 5

#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "LCD_2porty.c"
#include "I2C.c"
#include "SHTC3.c"
#include "EEPROM.c"

/*
	States of menu are in statusbyte_menu
	
	0	0x80	main menu
	1	0x81	setting time - ones of minutes
	2	0x82	setting time - tens of minutes
	3	0x83	setting time - ones of hours
	4	0x84	setting time - tens of hours
	5	0x85	setting date - tens of day
	6	0x86	setting date - ones of day
	7	0x87	setting date - month
	9	0x89	setting date - tens of year
	10	0x8A	setting date - ones of year
	11	0x8B	setting alarm time - ones of minutes
	12	0x8C	setting alarm time - tens of minutes
	13	0x8D	setting alarm time - ones of hours
	14	0x8E	setting alarm time - tens of hours
	15	0x8F	setting speaking enable
	16	0x90	setting voice instructions recognizing enable
	17	0x91	recording voice command to RV module
	18	0x92	setting day of week
	19	0x93	setting volume of speaking
*/

volatile char statusbyte_state = 0xFF;		// |ALARM|RTC_INT1|SW_TOP|SW5|SW4|SW3|SW2|SW1|		flags for internal functionality; set = 0, clear = 1
volatile char statusbyte_flags = 0x00;		// |FLAGS_SETTINGS_SPEAKING_ENABLED|FLAGS_ALARM_FIRST|FLAGS_RF_NOT_SAME|FLAGS_RF_FIRST|FLAGS_USART|FLAGS_SAY|FLAGS_MENU_LCD|FLAGS_GO_TO_MENU|
volatile char statusbyte_menu = 0x00;		// |MENU||||||||	flags of menu status
volatile char statusbyte_mask = 0xB0;		// |MASK_SW5|MASK_WRONG_DATE|MASK_LISTENING|MASK_SPEAKING|MASK_RH_T_OUT|MASK_RH_T_IN|MASK_DATE|MASK_ALARM|		alarm 1 = enabled, 0 = disabled; MASK_SW5 normal = 1; MASK_DATE, RH_T_IN... 1 = print actual value on LCD
volatile char statusbyte_sw = 0xFF;			// |1|1|SW_TOP|SW5|SW4|SW3|SW2|SW1|		flags of current state of switches; push = 0
volatile char statusbyte_last_sw = 0xFF;	// |1|1|SW_TOP|SW5|SW4|SW3|SW2|SW1|		flags of recent state of switches; push = 0
volatile char statusbyte_first = 0xFF;		// |1|1|SW_TOP|SW5|SW4|SW3|SW2|SW1|		flags if first time pressed 1 = yes; 0 = no
volatile char statusbyte_usart_sw;			//flags of voice command - equivalent of statusbyte_sw
volatile double mainTimer = 0;				//storing time in seconds from start of program
volatile double lightingTimer = 0;
volatile double rh_t_inTimer = 0;
volatile double switchesTimer = 0;
volatile double alarmTimer = 0;
volatile double resetTimer = 0;
volatile double FLAGS_USARTtimer = 0;
volatile char RTCdata[7] = {0, 0, 0, 0, 0, 0, 0};		//array for time and date data from RTC
volatile char timeValue[4];								//array for time data
volatile char dateValue[7];								//array for date data
volatile char RTClastDate;								//last date from RTC
volatile char MenuDataArray[6];							//array for setting values in menu
volatile char USARTreceived;							//received byte from USART
volatile char RVsettingStatus = 0;						//status of setting recognition of voice module
volatile unsigned char outdoorSensor[6];				//verified data from outdoor sensor
volatile char RFbuffer[8];								//buffer for received data - without validation
volatile unsigned char RFreceivedData[3][6] = {			//buffer for received data from outdoor sensor - received successfully
	{0xFF,0, 0, 0, 0, 0},
	{0xFF,0, 0, 0, 0, 0},
	{0xFF,0, 0, 0, 0, 0}
};
volatile char RFcounter = 0;							//counters for RF receiving bytes
volatile char RFrepeatsCounter;
volatile char volume;									//volume of speaking
volatile float temperatureIn;							//indoor temperature
volatile float humidityIn;								//indoor humidity

void USART_Init(void);									//prototypes of functions
void USART_Transmit(unsigned char data);
void USART_Flush(void);
void USART_Read(void);
void RVcommand(char RVcommandData);
void RVsend(char RVsendData);
void WT588Dcommand(char WT588Ddata);
void WT588Dsay(char WT588DsayAddr);
int SHR(char SHRbyte);
void dataImput(void);
void functions(void);
void statusMenuSw1(void);
void statusMenuSw2(void);
void statusMenuSw3(void);
void statusMenuSw4(void);
void statusMenuSw5(void);
void statusTime(void);
void statusDate(void);
void statusRH_T_in(void);
void statusRH_T_out(void);
void statusMenuIn(void);
void LCDclear4thLine(void);
void statusMenuOut(void);
void clearMenuDataArray(void);
void SetVoiceRecognizing(void);
void speaking(void);
void sayTime(char *sayTimeBuffer);
void sayDate(char *sayDateBuffer);
void sayRH_T(float sayRH_T_temp, char sayRH_T_hum);
void saySetting(void);

int main(void){
	DDRB = 0b00011111;						//XTAL|WT_bussy|WT_data|4 LCD data lines
	DDRC = 0b00000101;						//I2C lines|NC|SHR_LD|SHR_DATA|SHR_CLK
	DDRD = 0xF0;							//PD7-LCD_light|3 LCD command lines|INT1|INT0|2 USART lines
	PORTD = 1<<PORTD3;						//pull-up INT1
	PORTC = 1<<SHR_LD;
	EICRA |= ((1<<ISC11) | (1<<ISC10)) | ((1<<ISC01) | (1<<ISC00));	//set trigging external interrupts on rising edge
	EIMSK |= (1<<INT1) | (1<<INT0);									//enable external interrupts
	
	EEPROM_init();							//initialization of EEPROM
	USART_Init();							//initialization of USART
	LCDinit();								//initialization of LCD
	TWIinitMaster(8);						//initialization of I2C protocol
	_delay_ms(250);							//delay for stabilizing 3V3 for SHTC3
	SHTC3sleep();							//sleep temperature and humidity sensor
	
	statusbyte_mask = EEPROM_read(0x00);	//reload old settings
	volume = EEPROM_read(0x01);				//reload old volume
	statusbyte_mask &= 0xB1;				//use only flag bits of alarm, speaking and voice recognizing enable
	
	TWIsendStart(DS3231_ADDR);				//set output 1Hz on RTC for interrupt INT1 and set alarm 1 when hours, minutes and seconds matched
	TWIsendByte(0x07);						//set register for setting alarm 1 in ds3231 (4 bytes)
	TWIsendByte(0x00);						//
	TWIsendByte(EEPROM_read(0x02));			//minutes from permanent memory
	TWIsendByte(EEPROM_read(0x03));			//hours
	TWIsendByte(0x80);
	TWIsendByte(0x00);						//set alarm 2 in ds3231 (3 bytes) not used
	TWIsendByte(0x00);
	TWIsendByte(0x00);
	TWIsendByte(0x00);						//set 1Hz output
	TWIstop();
	_delay_us(10);
	TWIsendStart(DS3231_ADDR);				//clear alarm flag in DS3231
	TWIsendByte(0x0F);
	TWIsendByte(0x00);
	TWIstop();
	
	RVcommand(RV_WAIT);						//set RV module
	USART_Flush();							//clears FIFO USART register
	USARTreceived = 0xFF;
	char RVsetCounter = 5;
	while((USARTreceived == 0xFF) && (RVsetCounter > 0)){			//while didn't received feedback from RV module
		RVsend(0x24);						//send instruction which will return if sound instructions are recorded in RV module
		USART_Read();						//wait for feedback from RV module
		_delay_ms(30);
		RVsetCounter--;
	}
	if(USARTreceived == 0x01){				//if sound instructions are recorded
		RVcommand(RV_IMPORT_INSTRUCTIONS);	//start recognizing sound instructions
	}
	USARTreceived = NULL;					//clears variable for further purpose	
	WT588Dcommand(volume + 0xE0);			//send volume
	LCDgoto(4,0);							//intro
	LCDprintString("METEOSTANICA");
	LCDgoto(2,1);
	LCDprintString("PRE  NEVIDIACICH");
	LCDgoto(2, 3);
	LCDprintString("MATEJ GREGA 2020");
	WT588Dsay(WT_INTRO);
	LCD_PORT_COMMAND |= 1<<LCD_LIGHT;
	_delay_ms(4000);
	LCD_PORT_COMMAND &= ~(1<<LCD_LIGHT);
	LCDsendCommand(LCD_CLEAR);
	sei();
	
	while(1){								//endless loop
		dataImput();						//update switches and time, temperature, humidity values
		functions();						//printing info on LCD + menu
		speaking();
		_delay_ms(5);
    }
}

ISR(INT0_vect){								//received data from outdoor sensor - RF module
	cli();
	char RFreceived;
	RFreceived = SHR(SHR_RECEIVED) >> 4;	
	if(RFreceived < RF_START){
		RFbuffer[RFcounter] = RFreceived;
	}
	else{
		switch(RFreceived){
			case RF_NEW_DATA:
				RFrepeatsCounter = 0;
				for(unsigned char i = 0; i < 3; i++){
					RFreceivedData[i][0] = 0xFF;
				}
				break;
			case RF_START:
				RFcounter = 1;
				break;
			case RF_CONTINUE:
				if(!(statusbyte_flags & (1<<FLAGS_RF_FIRST))){
					statusbyte_flags |= 1<<FLAGS_RF_FIRST;
					RFcounter++;
				}
				break;
			case RF_STOP:
				if(RFcounter == 6){
					statusbyte_mask |= 1<<MASK_RH_T_OUT;
					for(char i = 0; i < 6; i++){
						RFreceivedData[RFrepeatsCounter][i] = RFbuffer[i+1];
					}
				}
				RFcounter = 0;
				if(RFrepeatsCounter < 2){
					RFrepeatsCounter++;
				}
				else{
					RFrepeatsCounter = 0;
				}
				break;
		}
	}
	if(RFreceived != RF_CONTINUE){
		statusbyte_flags &= ~(1<<FLAGS_RF_FIRST);
	}
	if(RFcounter >= 7){
		RFcounter = 0;
	}
	sei();
}

ISR(INT1_vect){								//1Hz from RTC
	statusbyte_state &= ~(1<<RTC_INT1);		//set flag of time update
}

void USART_Init(void){						//initialization of USART
	UBRR0H = 0x00;							//setting baud
	UBRR0L = 0x67;							//set to 9600
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);			//enable receiver and transmitter (1<<RXEN)|(1<<TXEN);
	UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);		//asynchronous, 8bit, 1 stop bit
}

void USART_Transmit(unsigned char data){	//send byte via USART
	while(!( UCSR0A & (1<<UDRE0)))			//wait while previous byte isn't sent yet
	;
	UDR0 = data;							//sent byte = write byte to UDR0
}

void USART_Flush(void){						//clear FIFO register - USART received bytes
	unsigned char USART_FlushData;
	while(UCSR0A & (1<<RXC0)) USART_FlushData = UDR0;	//while are some data in FIFO register, read them
}

void USART_Read(void){						//wait till USART data received and store byte to variable USARTreceived
	char USART_Read_counter = USART_READ_TIMEOUT;
	char USART_Read_valid = 1;
	while((!(UCSR0A & (1<<RXC0))) && (USART_Read_counter > 0)){			//while USART FIFO register is empty and waiting for receiving data is less than USART_READ_TIMEOUT
		_delay_ms(1);
		USART_Read_counter--;
		if(! USART_Read_counter){
			USART_Read_valid = 0;
		}
	}										//if received data or function hasn't received data in USART_READ_TIMEOUT duration
	if(USART_Read_valid){
		USARTreceived = UDR0;
	}
}

void RVsend(char RVsendData){				//send data to recognizing of voice module
	USART_Transmit(0xAA);					//send "head" means next byte is instruction for RV module
	USART_Transmit(RVsendData);				//instruction for RV module
}

void RVcommand(char RVcommandData){			//send instruction to RV module which successful feedback is 0xCC
	USART_Flush();							//clear FIFO USART register
	while(USARTreceived != 0xCC){			//while not received feedback of successfully send instruction
		RVsend(RVcommandData);				//send instruction
		USART_Read();						//wait for feedback
		_delay_ms(30);
	}
	USARTreceived = NULL;
	_delay_ms(30);
}

void WT588Dcommand(char WT588Ddata){		//sending address of .wav file to be played or command
	WT_PORT &= ~(1<<WT_DATA);				//communication trough one pin based on WT588D datasheet
	_delay_ms(5);
	cli();
	for(char WT588Dcounter = 0; WT588Dcounter < 8; WT588Dcounter++){
		WT_PORT |= 1<<WT_DATA;
		if(WT588Ddata & (1<<WT588Dcounter)){
			_delay_us(600);
			WT_PORT &= ~(1<<WT_DATA);
			_delay_us(200);
		}
		else{
			_delay_us(200);
			WT_PORT &= ~(1<<WT_DATA);
			_delay_us(600);
		}
	}
	WT_PORT |= 1<<WT_DATA;
	sei();
	_delay_ms(200);							//wait until WT588D set busy pin
}

void WT588Dsay(char WT588DsayAddr){
	int WT588Dsay_counter = WT588D_WAITING_TIMEOUT;
	while((WT_PIN & (1<<WT_BUSSY)) && (WT588Dsay_counter > 0)){				//wait if WT588D is speaking now
		_delay_ms(1);
		WT588Dsay_counter--;
	}
	WT588Dcommand(WT588DsayAddr);			//send address of .wav file to be played
}

int SHR(char SHRbyte){						//return value from shift registers SHRbyte 1 = switches, 2 = received data433MHz
	char SHRresult;
	SHR_PORT &= ~(1<<SHR_LD);				//load current inputs into shift register
	_delay_us(1);
	SHR_PORT |= 1<<SHR_LD;
	cli();
	while(SHRbyte){
		SHRresult = 0;
		for(char SHRcounter = 8; SHRcounter > 0; SHRcounter--){
			_delay_us(1);
			if(SHR_PIN & (1<<SHR_DATA)){
				SHRresult |= 1<<(SHRcounter - 1);
			}
			SHR_PORT |= 1<<SHR_CLK;
			_delay_us(1);
			SHR_PORT &= ~(1<<SHR_CLK);
		}
		SHRbyte--;
	}
	sei();
	return SHRresult;
}

void LCDclear4thLine(void){					//clear 4th line and set cursor to first position in 4th line
	LCDgoto(0, 3);
	LCDprintString("                    ");
	LCDgoto(0, 3);
}

void clearMenuDataArray(void){				//clear array for menu setting data
	MenuDataArray[0] = 0;
	MenuDataArray[1] = 0;
	MenuDataArray[2] = 0;
	MenuDataArray[3] = 0;
	MenuDataArray[4] = 0;
	MenuDataArray[5] = 0;
}

void dataImput(void){
	static char alarm_volume = 1;
	statusbyte_sw = SHR(SHR_SWITCHES);			//switches update
	if((UCSR0A & (1<<RXC0)) || (statusbyte_flags & (1<<FLAGS_USART))){		//if received data form RV module or data haven't been said yet
		if(!(statusbyte_flags & (1<<FLAGS_USART))){	//if received data in this program cycle; means first time
			statusbyte_usart_sw = UDR0;			//read byte
		}
		statusbyte_flags |= 1<<FLAGS_USART;		//set flag of first time
												//if received byte is information of voice command and voice command are enabled and program isn't in menu
		if(((statusbyte_usart_sw >= 0x11) && (statusbyte_usart_sw <= 0x15)) && ((statusbyte_mask & (1<<MASK_LISTENING)) && (!(statusbyte_menu & (1<<MENU))))){
			switch(statusbyte_usart_sw){
				case 0x11:						//equivalent of switch 1
					statusbyte_sw = ~(1<<SW1);	//set flag of pressed switch
					break;
				case 0x12:						//equivalent of switch 2
					statusbyte_sw = ~(1<<SW2);
					break;
				case 0x13:						//equivalent of switch 3
					statusbyte_sw = ~(1<<SW3);
					break;
				case 0x14:						//equivalent of switch 4
					statusbyte_sw = ~(1<<SW4);
					break;
				case 0x15:						//equivalent of switch 5
					statusbyte_sw = ~(1<<SW5);
					statusbyte_flags &= ~(1<<FLAGS_USART);	//when change of alarm enable clear flag of more cycles because this switch is also menu
					break;
			}
		}
	}
	statusbyte_state &= statusbyte_sw & statusbyte_last_sw;					//set only 6 LSB bits, don't change INT1_RTC and ALARM
	statusbyte_state |= (statusbyte_sw & statusbyte_last_sw) & 0x3F;		//set only 6 LSB bits, don't change INT1_RTC and ALARM
	
	
	if((!(statusbyte_sw & (1<<SW4))) && (statusbyte_first & (1<<SW4))){	//if switch 5 is pressed for the first time and flag of first time is 1
		resetTimer = mainTimer + 10;			//after 5 seconds switches timer and main timer will match = switch 5 was pressed for 5 seconds
	}
		
	
	if((!(statusbyte_sw & (1<<SW5))) && (statusbyte_mask & (1<<MASK_SW5))){	//if switch 5 is pressed for the first time and flag of first time is 1
		statusbyte_mask &= ~(1<<MASK_SW5);		//set flag of first time to 0
		switchesTimer = mainTimer + 5;			//after 5 seconds switches timer and main timer will match = switch 5 was pressed for 5 seconds
	}
	if(statusbyte_sw & (1<<SW5)){				//if switch 5 isn't pressed
		statusbyte_mask |= 1<<MASK_SW5;			//set flag of first time to 1
		switchesTimer = 0;						//clearing switches timer makes that it won't match with main timer
	}
	
	if(FLAGS_USARTtimer != mainTimer){			//if error in receiving data from RV module occurred
		statusbyte_flags &= ~(1<<FLAGS_USART);	//clear flag
	}
/*----------------------------------------------------------------------------------------------*/
	if(!(statusbyte_state & (1<<RTC_INT1))){	//if interrupt from RTC (1Hz) occurred
		statusbyte_state |= 1<<RTC_INT1;		//clear flag
		mainTimer++;							//increment main timer storing time in seconds from beginning of program
		RTClastDate = RTCdata[4];				//save last date to know if will occur change of date
		TWIsendStart(DS3231_ADDR);				//go to register address 0x00
		TWIsendByte(0x00);
		TWIstop();
		TWIreceive(DS3231_ADDR, RTCdata, 7);				//read 7 bytes (3 time bytes, 1 not used byte(day), 3 date bytes)
		timeValue[HOURS_TENS] = (RTCdata[2] & 0x30)>>4;		//tens of hours
		timeValue[HOURS_ONES] = RTCdata[2] & 0x0F;			//ones of hours
		timeValue[MINUTES_TENS] = (RTCdata[1] & 0x70)>>4;	//tens of minutes
		timeValue[MINUTES_ONES] = RTCdata[1] & 0x0F;		//ones of minutes
		dateValue[DAY_TENS] = (RTCdata[4] & 0x30)>>4;		//tens of day
		dateValue[DAY_ONES] = RTCdata[4] & 0x0F;			//ones of day
		dateValue[MONTH_TENS] = (RTCdata[5] & 0x10)>>4;		//tens of month
		dateValue[MONTH_ONES] = RTCdata[5] & 0x0F;			//ones of month
		dateValue[YEAR_TENS] = (RTCdata[6] & 0xF0)>>4;		//tens of years
		dateValue[YEAR_ONES] = RTCdata[6] & 0x0F;			//ones of years
		if(!(statusbyte_menu & (1<<MENU))){					//if state of device is not menu, then print time on LCD
			LCDgoto(0, 0);									//print time on LCD as big number (3 lines)
			LCDbigDigit(timeValue[3]);
			LCDbigDigit(timeValue[2]);
			LCDbigDigit(10);
			LCDbigDigit(timeValue[1]);
			LCDbigDigit(timeValue[0]);
			LCDbigDigit(10);
			LCDbigDigit((RTCdata[0] & 0x70)>>4);
			LCDbigDigit(RTCdata[0] & 0x0F);
		}
		if(RTClastDate != RTCdata[4]){						//if new date should be print on LCD
			statusbyte_mask |= 1<<MASK_DATE;				//set flag
		}
		TWIsendStart(DS3231_ADDR);							//go to register address 0x0F (alarm)
		TWIsendByte(0x0F);
		TWIstop();
		TWIreceive(DS3231_ADDR, RTCdata, 1);				//receive information of alarm (active(1) or not(0) (LSB))
		if(RTCdata[0] & 1){									//if alarm
			if(statusbyte_mask & (1<<MASK_ALARM)){			//if alarm enabled
				statusbyte_state &= ~(1<<ALARM);			//set alarm flag(0)
				if(!(statusbyte_flags & (1<<FLAGS_ALARM_FIRST))){	//if it's alarm change from idle to active
					statusbyte_flags |= 1<<FLAGS_ALARM_FIRST;	//set flag 
					alarmTimer = mainTimer + 300;				//add 5min to to alarm timer so alarm will stop after 5min automatically
				}
			}
			else{											//if alarm disabled
				statusbyte_state |= 1<<ALARM;				//clear alarm flag
				TWIsendStart(DS3231_ADDR);					//clear alarm flag in DS3231
				TWIsendByte(0x0F);
				TWIsendByte(0x00);
				TWIstop();
				statusbyte_flags &= ~(1<<FLAGS_ALARM_FIRST);
			}
		}
		
		if(!(statusbyte_flags & (1<<FLAGS_USART))){			//increment
			FLAGS_USARTtimer = mainTimer;
		}
	}
/*----------------------------------------------------------------------------------------------*/
	if((!(statusbyte_sw & (1<<SW_TOP))) || ((alarmTimer <= mainTimer) && (!(statusbyte_state & (1<<ALARM))))){			//if top switch is pressed or alarm is ringing for 5 min
		lightingTimer = mainTimer + 15;			//add 15 seconds to lighting timer makes lighting turned on for 15 seconds
		if(!(statusbyte_state & (1<<ALARM))){	//if alarm
			statusbyte_state |= 1<<ALARM;		//clear alarm flag
			statusbyte_flags &= ~(1<<FLAGS_ALARM_FIRST);	//clear flag for alarm timer
			WT588Dcommand(WT588D_STOP);			//stop playing alarm
			TWIsendStart(DS3231_ADDR);			//clear alarm flag in DS3231
			TWIsendByte(0x0F);
			TWIsendByte(0x00);
			TWIstop();
			alarm_volume = 1;					//reset alarm starting volume to minimum
			WT588Dcommand(volume + 0xE0);		//set volume for normal operation
		}
	}
/*----------------------------------------------------------------------------------------------*/
	if(lightingTimer >= mainTimer){				//if LCD lighting should be turned on
		LCD_PORT_COMMAND |= 1<<LCD_LIGHT;		//turn LCD lighting on
	}
	if((lightingTimer < mainTimer) && (!(statusbyte_menu & (1<<MENU)))){
		LCD_PORT_COMMAND &= ~(1<<LCD_LIGHT);	//turn LCD lighting off
	}
/*----------------------------------------------------------------------------------------------*/
	if(rh_t_inTimer <= mainTimer){							//if humidity and temperature measurement should occur (every 30 seconds)
		temperatureIn = 0;									//clear actual values
		humidityIn = 0;
		for(char SHTC3counter = 10; SHTC3counter > 0; SHTC3counter--){
			 SHTC3measure();								//measure RH and T
			 temperatureIn += SHTC3_Temperature;
			 humidityIn += SHTC3_Humidity;
			 _delay_ms(1);
		}
		temperatureIn /= 10;								//make average
		humidityIn /= 10;
		rh_t_inTimer = mainTimer + 30;			//add 30seconds to timer so next measurement will be after 30 seconds
		statusbyte_mask |= 1<<MASK_RH_T_IN;		//set flag of printing new value on LCD 
	}
/*----------------------------------------------------------------------------------------------*/
	if((!(statusbyte_state & (1<<ALARM))) && (!(WT_PIN & (1<<WT_BUSSY)))){	//if alarm and WT588D isn't speaking
		WT588Dcommand(alarm_volume + 0xE0);		//set volume
		WT588Dcommand(0x54);					//play alarm melody
		if(alarm_volume < 7){					//if volume is lower than maximum
			alarm_volume++;						//increment volume so next time alarm will be louder
		}
	}
}

void functions(void){					//displaying required information on LCD, menu setting
	if(statusbyte_menu & (1<<MENU)){	//if program is in menu
		switch(statusbyte_sw | 0xE0){	//depends on actual state of switches
			case 0b11111110:			//pressed switch 1
				statusMenuSw1();
				break;
			case 0b11111101:			//pressed switch 2 ...
				statusMenuSw2();
				break;
			case 0b11111011:
				statusMenuSw3();
				break;
			case 0b11110111:
				statusMenuSw4();
				break;
			case 0b11101111:
				statusMenuSw5();
				break;
			default:										//if none of switches is pressed
				statusbyte_last_sw = statusbyte_sw;			//move actual state of switches to last state of switches
				statusbyte_first = 0xFF;					//clear all flags that switch was already pressed
		}
/*----------------------------------------------------------------------------------------------*/
		if(statusbyte_flags & (1<<FLAGS_MENU_LCD)){			//(menu) if flag of LCD update is set
			statusbyte_flags &= ~(1<<FLAGS_MENU_LCD);		//clear flag
			if(statusbyte_menu == (1<<MENU)){				//if main menu
				LCDsendCommand(LCD_CLEAR);					//print menu screen on LCD
				LCDwrite('1');
				LCDwrite(0b01111110);
				LCDprintString(" CAS A DATUM");
				LCDgoto(0, 1);
				LCDwrite('2');
				LCDwrite(0b01111110);
				LCDprintString(" BUDIK");
				LCDgoto(0, 2);
				LCDwrite('3');
				LCDwrite(0b01111110);
				LCDprintString(" ZVUK A PRIKAZY");
				LCDgoto(0, 3);
				LCDwrite('4');
				LCDwrite(0b01111110);
				LCDprintString(" NAHOVOR PRIKAZY");
			}
/*----------------------------------------------------------------------------------------------*/
			if((statusbyte_menu >= 0x81) && (statusbyte_menu <= 0x84)){	//(menu)if setting time
				LCDgoto(4, 1);								//print screen of setting time
				LCDprintString("NASTAVTE CAS");
				LCDgoto(7, 2);
				LCDputInt(MenuDataArray[3]);				//actual values of setting time
				LCDputInt(MenuDataArray[2]);
				LCDwrite(':');
				LCDputInt(MenuDataArray[1]);
				LCDputInt(MenuDataArray[0]);
				switch(statusbyte_menu){					//set blinking cursor to position which depends on digit being set
					case 1 | (1<<MENU):
						LCDgoto(11, 2);
						break;
					case 2 | (1<<MENU):
						LCDgoto(10, 2);
						break;
					case 3 | (1<<MENU):
						LCDgoto(8, 2);
						break;
					case 4 | (1<<MENU):
						LCDgoto(7, 2);
						break;
				}
				if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
					sayTime(MenuDataArray);
				}
			}
/*----------------------------------------------------------------------------------------------*/
			if((statusbyte_menu >= 0x85) && (statusbyte_menu <= 0x8A)){	//(menu) if setting date
				LCDgoto(3, 1);								//print screen of setting date
				LCDprintString("NASTAVTE DATUM");
				LCDgoto(5, 2);
				LCDputInt(MenuDataArray[0]);				//actual values of setting date
				LCDputInt(MenuDataArray[1]);
				LCDwrite('.');
				LCDputInt(MenuDataArray[2]);
				LCDprintString(".20");
				LCDputInt(MenuDataArray[4]);
				LCDputInt(MenuDataArray[5]);
				LCDwrite(' ');								//print space to erase last digit when month changes from two-digits to one digit
				switch(statusbyte_menu){					//set blinking cursor to position which depends on digit being set
					case 5 | (1<<MENU):
						LCDgoto(5, 2);
						break;
					case 6 | (1<<MENU):
						LCDgoto(6, 2);
						break;
					case 7 | (1<<MENU):
						LCDgoto(8, 2);
						break;
					case 9 | (1<<MENU):
						if(MenuDataArray[2] > 9){
							LCDgoto(13, 2);
						}
						else{
							LCDgoto(12, 2);
						}
						break;
					case 10 | (1<<MENU):
						if(MenuDataArray[2] > 9){
							LCDgoto(14, 2);
						}
						else{
							LCDgoto(13, 2);
						}
						break;
				}
				if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
					sayDate(MenuDataArray);
				}
			}
/*----------------------------------------------------------------------------------------------*/
			if((statusbyte_menu >= 0x8B) && (statusbyte_menu <= 0x8E)){	//(menu) if setting time for alarm 
				LCDgoto(3, 1);								//same as setting time except words "ALARM SETTING"
				LCDprintString("NASTAVTE BUDIK");
				LCDgoto(7, 2);
				LCDputInt(MenuDataArray[3]);
				LCDputInt(MenuDataArray[2]);
				LCDwrite(':');
				LCDputInt(MenuDataArray[1]);
				LCDputInt(MenuDataArray[0]);
				switch(statusbyte_menu){
					case 11 | (1<<MENU):
					LCDgoto(11, 2);
					break;
					case 12 | (1<<MENU):
					LCDgoto(10, 2);
					break;
					case 13 | (1<<MENU):
					LCDgoto(8, 2);
					break;
					case 14 | (1<<MENU):
					LCDgoto(7, 2);
					break;
				}
				if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
					sayTime(MenuDataArray);
				}
			}
/*----------------------------------------------------------------------------------------------*/
			if(statusbyte_menu == 0x8F){					//(menu) if setting speaking enable
				LCDgoto(5, 1);								//print screen of setting speaking enable
				LCDprintString("HOVORENIE:");
				LCDgoto(6, 2);
				if(statusbyte_flags & (1<<FLAGS_SETTINGS_SPEAKING_ENABLED)){	//if speaking is enabled 
					LCDprintString("ZAPNUTE");
					if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
						WT588Dsay(WT_ZAPNUTE);
					}
				}
				else{											//if speaking is disabled 
					LCDprintString("VYPNUTE");
					if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
						WT588Dsay(WT_VYPNUTE);
					}
				}
			}
/*----------------------------------------------------------------------------------------------*/
			if(statusbyte_menu == 0x90){						//(menu) if setting voice instructions recognizing enable
				LCDsendCommand(LCD_CLEAR);
				LCDgoto(4, 0);
				LCDprintString("ROZPOZNAVANIE");
				LCDgoto(6, 1);
				LCDprintString("PRIKAZOV:");
				LCDgoto(6, 2);
				if(statusbyte_mask & (1<<MASK_LISTENING)){		//if enabled
					LCDprintString("ZAPNUTE");
					if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
						WT588Dsay(WT_ZAPNUTE);
					}
				}
				else{											//if disabled
					LCDprintString("VYPNUTE");
					if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
						WT588Dsay(WT_VYPNUTE);
					}
				}
			}
/*----------------------------------------------------------------------------------------------*/
			if(statusbyte_menu == 0x91){		//(menu) if setting voice recognizing
				SetVoiceRecognizing();
			}
/*----------------------------------------------------------------------------------------------*/
			if(statusbyte_menu == 0x92){		//(menu) if setting day of week
				LCDgoto(4, 1);
				LCDprintString("NASTAVTE DEN:");
				LCDgoto(6, 2);
				switch(MenuDataArray[0]){					//print day of week
					case 1:
					LCDprintString("PONDELOK");
					break;
					case 2:
					LCDprintString("UTOROK  ");
					break;
					case 3:
					LCDprintString("STREDA  ");
					break;
					case 4:
					LCDprintString("STVRTOK ");
					break;
					case 5:
					LCDprintString("PIATOK  ");
					break;
					case 6:
					LCDprintString("SOBOTA  ");
					break;
					case 7:
					LCDprintString("NEDELA  ");
					break;
				}
				if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
					WT588Dsay(MenuDataArray[0] + (WT_PONDELOK - 1));
				}
			}
/*----------------------------------------------------------------------------------------------*/
			if(statusbyte_menu == 0x93){					//(menu) if setting volume
				LCDgoto(4, 1);
				LCDprintString("HLASITOST: ");				//print volume information
				LCDputInt(volume);
				LCDgoto(6, 2);
				LCDprintString("-------");
				LCDgoto(6, 2);
				for(char counter = volume; counter > 0; counter--){
					LCDwrite(0xFF);
				}
				if(statusbyte_mask & (1<<MASK_SPEAKING)){			//if speaking enabled
					WT588Dcommand(volume + 0xE0);					//set volume to actual setting
					WT588Dsay(WT_HLASITOST);
//					WT588Dsay(WT_JE);
					WT588Dsay(volume);
				}
			}
		}
	}
/*----------------------------------------------------------------------------------------------*/
	else{										//if program is not in menu
		switch(statusbyte_state | 0xE0){		//depends on which switch was pressed as last
			case 0b11111110:					//switch 1
				statusTime();
				break;
			case 0b11111101:					//switch 2 ...
				statusDate();
				break;
			case 0b11111011:
				statusRH_T_in();
				break;
			case 0b11110111:
				statusRH_T_out();
				break;
			case 0b11101111:
				statusMenuIn();
				break;
			default:								//if more than one switch was pressed  e.g. one was pressed before and one is pressed now = change of information on LCD
				statusbyte_last_sw = statusbyte_sw;	//move actual values of switches to last so in next loop program will go to one of switch cases
				statusbyte_first = 0xFF;
		}
	}
}

void statusMenuSw1(void){											//all operations which depends on switch 1 in menu
	if(statusbyte_first & (1<<SW1)){								//if this is first program cycle in which program go trough this function
		statusbyte_first = ~(1<<SW1);								//clear flag of first time so next operations are done only once per switch press
		statusbyte_flags |= 1<<FLAGS_MENU_LCD;						//set flag of LCD update
		WT588Dcommand(WT588D_STOP);
		if((statusbyte_menu >= 0x81) && (statusbyte_menu <= 0x84)){	//if setting time
			if(MenuDataArray[(statusbyte_menu & 0x0F) - 1] > 0){	//if value of actual digit being set is not zero (is greater)
				MenuDataArray[(statusbyte_menu & 0x0F) - 1]--;		//decrement value of actual digit being set
			}
		}
		if(((statusbyte_menu >= 0x85) && (statusbyte_menu <= 0x8A)) && (statusbyte_menu != 0x87)){	//if setting date except setting month
			if(MenuDataArray[(statusbyte_menu & 0x0F) - 5] > 0){	//if value of actual digit being set is not zero (is greater)
				MenuDataArray[(statusbyte_menu & 0x0F) - 5]--;		//decrement value of actual digit being set
			}
		}
		if((statusbyte_menu >= 0x8B) && (statusbyte_menu <= 0x8E)){	//if setting time of alarm
			if(MenuDataArray[(statusbyte_menu & 0x0F) - 11] > 0){	//...
				MenuDataArray[(statusbyte_menu & 0x0F) - 11]--;
			}
		}
		switch(statusbyte_menu){									//other states of statusbyte_menu
			case (1<<MENU):											//if main menu
				statusbyte_menu = 1 | (1<<MENU);					//set flags of setting time specifically ones of minutes
				if(statusbyte_mask & (1<<MASK_SPEAKING)){			//if speaking enabled
					saySetting();
				}
				clearMenuDataArray();
				LCDsendCommand(LCD_CLEAR);
				LCDsendCommand(0x0D);								//blinking cursor on LCD
				break;
			case 7 | (1<<MENU):
				if(MenuDataArray[2] > 1){
					MenuDataArray[2]--;
				}
				break;
			case 15 | (1<<MENU):									//if setting speaking enable
				statusbyte_flags &= ~(1<<FLAGS_SETTINGS_SPEAKING_ENABLED);				//disable speaking
				break;
			case 16 | (1<<MENU):									//if setting recognizing of voice commands enable
				statusbyte_mask &= ~(1<<MASK_LISTENING);			//disable recognizing commands
				break;
			case 18 | (1<<MENU):									//if setting day of week
				if(MenuDataArray[0] > 1){							//if day of week is greater than 1 (Monday)
					MenuDataArray[0]--;								//decrement day of week
				}
				break;
			case 19 | (1<<MENU):									//if setting volume
				if(volume > 0){										//if volume is greater than 0
					volume--;										//decrement volume
				}
				break;
		}
	}
}

void statusMenuSw2(void){											//all operations which depends on switch 2 in menu
	if(statusbyte_first & (1<<SW2)){								//if this is first program cycle in which program go trough this function
		statusbyte_first = ~(1<<SW2);								//clear flag of first time so next operations are done only once per switch press
		statusbyte_flags |= 1<<FLAGS_MENU_LCD;						//set flag of LCD update
		WT588Dcommand(WT588D_STOP);
																	//if setting date, tens of day, tens of month, tens of year or ones of year
		if(((statusbyte_menu == 0x86) || (statusbyte_menu == 0x88)) || ((statusbyte_menu == 0x89) || (statusbyte_menu == 0x8A))){
			if(MenuDataArray[(statusbyte_menu & 0x0F) - 5] < 9){	//if value of actual digit being set is less than 9
				MenuDataArray[(statusbyte_menu & 0x0F) - 5]++;		//increment value of actual digit being set
			}
		}
		if((statusbyte_menu == 0x81) || (statusbyte_menu == 0x83)){	//if setting time, ones of minutes or ones of hours
			if(MenuDataArray[(statusbyte_menu & 0x0F) - 1] < 9){	//if value of actual digit being set is less than 9
				MenuDataArray[(statusbyte_menu & 0x0F) - 1]++;		//increment value of actual digit being set
			}
		}
		if((statusbyte_menu == 0x82) || (statusbyte_menu == 0x8C)){	//if setting tens of minutes of time or time of alarm 
			if(MenuDataArray[1] < 5){								//if tens of minutes is less than 5
				MenuDataArray[1]++;									//increment tens of minutes
			}
		}
		if((statusbyte_menu == 0x84) || (statusbyte_menu == 0x8E)){	//if setting tens of hours of time or time of alarm
			if(MenuDataArray[3] < 2){								//if tens of hours is less than 2
				MenuDataArray[3]++;									//increment tens of hours
			}
		}
		if((statusbyte_menu == 0x8B) || (statusbyte_menu == 0x8D)){	//if setting time of alarm, ones of minutes or ones of hours
			if(MenuDataArray[(statusbyte_menu & 0x0F) - 11] < 9){	//if value of actual digit being set is less than 9
				MenuDataArray[(statusbyte_menu & 0x0F) - 11]++;		//increment value of actual digit being set
			}
		}
		switch(statusbyte_menu){
			case (1<<MENU):											//if main menu
				statusbyte_menu = 11 | (1<<MENU);					//set flags of setting alarm specifically ones of minutes
				if(statusbyte_mask & (1<<MASK_SPEAKING)){			//if speaking enabled
					saySetting();
				}
				clearMenuDataArray();
				LCDsendCommand(LCD_CLEAR);
				LCDsendCommand(0x0D);								//blinking cursor on LCD
				break;
			case 5 | (1<<MENU):										//setting date, tens of day
				if(MenuDataArray[0] < 3){							//if tens of day is less than 3
					MenuDataArray[0]++;								//increment tens of day
				}
				break;
			case 7 | (1<<MENU):										//setting date, tens of month
				if(MenuDataArray[2] < 12){							//if tens of month is less than 1
					MenuDataArray[2]++;								//increment tens of month
				}
				break;
			case 15 | (1<<MENU):									//if setting speaking enable
				statusbyte_flags |= 1<<FLAGS_SETTINGS_SPEAKING_ENABLED;				//enable speaking
				break;
			case 16 | (1<<MENU):									//if setting recognizing of voice commands
				statusbyte_mask |= 1<<MASK_LISTENING;				//enable recognizing of voice commands
				break;
			case 18 | (1<<MENU):									//if setting day of week
				if(MenuDataArray[0] < 7){							//if day of week is less than 7 - Sunday
					MenuDataArray[0]++;								//increment day of week
				}
				break;
			case 19 | (1<<MENU):									//if setting volume of speaking
				if(volume < 7){										//if less than maximum - 7
					volume++;										//increment volume
				}
		}
	}
}

void statusMenuSw3(void){							//all operations which depends on switch 3 in menu
	if(statusbyte_first & (1<<SW3)){				//if this is first program cycle in which program go trough this function
		statusbyte_first = ~(1<<SW3);				//clear flag of first time so next operations are done only once per switch press
		statusbyte_flags |= 1<<FLAGS_MENU_LCD;		//set flag of LCD update
		WT588Dcommand(WT588D_STOP);
		switch(statusbyte_menu){
			case (1<<MENU):							//if main menu
				statusbyte_menu = 15 | (1<<MENU);	//set flags of setting speaking enable and recognizing of voice commands enable
				if(statusbyte_mask & (1<<MASK_SPEAKING)){			//if speaking enabled
					if(statusbyte_mask & (1<<MASK_SPEAKING)){
						statusbyte_flags |= 1<<FLAGS_SETTINGS_SPEAKING_ENABLED;
					}
					else{
						statusbyte_flags &= ~(1<<FLAGS_SETTINGS_SPEAKING_ENABLED);
					}
					saySetting();
				}
				LCDsendCommand(LCD_CLEAR);
				break;
		}
	}
}

void statusMenuSw4(void){							//all operations which depends on switch 4 in menu
	if(statusbyte_first & (1<<SW4)){				//if this is first program cycle in which program go trough this function
		statusbyte_first = ~(1<<SW4);				//clear flag of first time so next operations are done only once per switch press
//		statusbyte_flags |= 1<<FLAGS_MENU_LCD;		//set flag of LCD update
		WT588Dcommand(WT588D_STOP);
		switch(statusbyte_menu){
			case (1<<MENU):							//if main menu
				statusbyte_menu = 17 | (1<<MENU);	//set flags of recording voice commands
				statusbyte_flags |= 1<<FLAGS_MENU_LCD;		//set flag of LCD update
				break;
		}
	}
}

void statusMenuSw5(void){							//all operations which depends on switch 5 in menu
	if(statusbyte_first & (1<<SW5)){				//if this is first program cycle in which program go trough this function
		statusbyte_first = ~(1<<SW5);				//clear flag of first time so next operations are done only once per switch press
		statusbyte_flags |= 1<<FLAGS_MENU_LCD;		//set flag of LCD update
													//if setting time except tens of hours, date except month and ones of years, time of alarm except tens of hours, or setting speaking enable
		if(((((statusbyte_menu >= 0x81) && (statusbyte_menu <= 0x83)) || (((statusbyte_menu >= 0x85) && (statusbyte_menu <= 0x89)) && (statusbyte_menu != 0x87))) || ((statusbyte_menu >= 0x8B) && (statusbyte_menu <= 0x8D))) || (statusbyte_menu == 0x8F)){
			statusbyte_menu ++;						//increment menu state - from time, ones of minutes to tens of minutes ...
		}
		else{										//else state of menu
			switch(statusbyte_menu){
				case (1<<MENU):						//main menu
					statusMenuOut();				//go out of menu
					break;
				case 4 | (1<<MENU):													//end of setting time - tens of hours
					if((MenuDataArray[3] == 2) && ((MenuDataArray[2]) > 3)){		//if set time is not valid
						LCDsendCommand(LCD_CLEAR);									//print error message on LCD
						LCDgoto(5, 1);
						LCDprintString("CHYBNY CAS");
						if(statusbyte_mask & (1<<MASK_SPEAKING)){		//if speaking enabled
							WT588Dsay(WT_CHYBNY);
							WT588Dsay(WT_CAS);
						}
						_delay_ms(3000);
					}
					else{															//if set time is valid
						TWIsendStart(DS3231_ADDR);									//send new values to RTC
						TWIsendByte(0x00);
						TWIsendByte(0x00);
						TWIsendByte((MenuDataArray[1]<<4) | (MenuDataArray[0]));
						TWIsendByte((MenuDataArray[3]<<4) | (MenuDataArray[2]));
						TWIstop();
						if(statusbyte_mask & (1<<MASK_SPEAKING)){		//if speaking enabled
							WT588Dsay(WT_POTVRDENE);
							statusbyte_menu = 0;
							sayTime(MenuDataArray);
							_delay_ms(1000);
						}
					}
					statusbyte_menu = 5 |(1<<MENU);									//set flag of setting date - tens of day
					clearMenuDataArray();											//clear menu data array for setting date
					MenuDataArray[1] = 1;											//preset date 01.01.2020
					MenuDataArray[2] = 1;
					MenuDataArray[4] = 2;
					break;
				case 7 | (1<<MENU):
					statusbyte_menu = 9 | (1<<MENU);								//setting tens of years
					break;
				case 10 | (1<<MENU):												//end of setting date
					statusbyte_menu = 18 | (1<<MENU);								//go to setting day of week
					LCDsendCommand(0x0C);											//turn off blinking cursor
					char RTCday = (MenuDataArray[0] * 10) + MenuDataArray[1];		//volatiles with values of day, month and year not ones of day, tens of day...
					char RTCmonth = MenuDataArray[2];
					char RTCyear = (MenuDataArray[4] * 10) + MenuDataArray[5];
					//testing if set date is valid
					if(RTCday == 0){												//if day is zero
						statusbyte_mask |= 1<<MASK_WRONG_DATE;						//set not valid date flag
					}
					if((RTCmonth > 12) || (RTCmonth == 0)){							//testing if month is not valid
						statusbyte_mask |= 1<<MASK_WRONG_DATE;
					}
					if(RTCyear % 4){												//if not leap year
						if((RTCday > 28) && (RTCmonth == 2)){						//testing if day in February is not valid
							statusbyte_mask |= 1<<MASK_WRONG_DATE;
						}
					}
					else{															//if leap year
						if((RTCday > 29) && (RTCmonth == 2)){						//testing if day in February is not valid
							statusbyte_mask |= 1<<MASK_WRONG_DATE;
						}
					}
					if(!((RTCmonth + 1) % 2)){										//if even month
						if((RTCmonth <= 7) && (RTCday > 31)){						//testing if day is not valid - greater than 30 or 31 depends on month
							statusbyte_mask |= 1<<MASK_WRONG_DATE;
						}
						if((RTCmonth >= 8) && (RTCday > 30)){
							statusbyte_mask |= 1<<MASK_WRONG_DATE;
						}
					}
					else{															//if odd month
						if((RTCmonth <= 7) && (RTCday > 30)){
							statusbyte_mask |= 1<<MASK_WRONG_DATE;
						}
						if((RTCmonth >= 8) && (RTCday > 31)){
							statusbyte_mask |= 1<<MASK_WRONG_DATE;
						}
					}
					if(statusbyte_mask & (1<<MASK_WRONG_DATE)){						//if flag of not valid date is set
						statusbyte_mask &= ~(1<<MASK_WRONG_DATE);					//clear flag
						LCDsendCommand(LCD_CLEAR);									//print error message on LCD
						LCDgoto(4, 1);
						LCDprintString("CHYBNY DATUM");
						if(statusbyte_mask & (1<<MASK_SPEAKING)){		//if speaking enabled
							WT588Dsay(WT_CHYBNY);
							WT588Dsay(WT_DATUM);
						}
						_delay_ms(3000);
					}
					else{															//if date is valid
						MenuDataArray[2] = RTCmonth / 10;
						MenuDataArray[3] = RTCmonth % 10;
						TWIsendStart(DS3231_ADDR);									//send new date to RTC
						TWIsendByte(0x04);
						TWIsendByte((MenuDataArray[0]<<4) | MenuDataArray[1]);
						TWIsendByte((MenuDataArray[2]<<4) | MenuDataArray[3]);
						TWIsendByte((MenuDataArray[4]<<4) | MenuDataArray[5]);
						TWIstop();
					}
					LCDsendCommand(LCD_CLEAR);
					MenuDataArray[0] = 1;											//set menu data array to 1 for setting day of week (Monday)
					break;
				case 14 | (1<<MENU):												//if end of setting time of alarm - tens of hours
					LCDsendCommand(0x0C);											//turn off blinking cursor
					if((MenuDataArray[3] == 2) && ((MenuDataArray[2]) > 3)){		//if set time is not valid
						LCDsendCommand(LCD_CLEAR);									//print error message on LCD
						LCDgoto(5, 1);
						LCDprintString("CHYBNY CAS");
						if(statusbyte_mask & (1<<MASK_SPEAKING)){					//if speaking enabled
							WT588Dsay(WT_CHYBNY);
							WT588Dsay(WT_CAS);
						}
						_delay_ms(3000);
					}
					else{															//if set time is valid
						TWIsendStart(DS3231_ADDR);									//send new time of alarm to RTC
						TWIsendByte(0x07);
						TWIsendByte(0x00);
						TWIsendByte((MenuDataArray[1]<<4) | (MenuDataArray[0]));
						TWIsendByte((MenuDataArray[3]<<4) | (MenuDataArray[2]));
						TWIstop();
						EEPROM_write(0x02, (MenuDataArray[1]<<4) | (MenuDataArray[0]));	//store alarm time in permanent memory
						EEPROM_write(0x03, (MenuDataArray[3]<<4) | (MenuDataArray[2]));
						if(statusbyte_mask & (1<<MASK_SPEAKING)){					//if speaking enabled
							WT588Dsay(WT_POTVRDENE);
							statusbyte_menu &= ~(1<<MENU);
							sayTime(MenuDataArray);
							statusbyte_menu |= 1<<MENU;
						}
					}
					statusbyte_menu = 1<<MENU;										//go to main menu
					break;
				case 16 | (1<<MENU):												//if end of setting speaking enable and recognizing of voice instructions
					statusbyte_menu = 19 | (1<<MENU);								//go to setting volume
					if(statusbyte_flags & (1<<FLAGS_SETTINGS_SPEAKING_ENABLED)){
						statusbyte_mask |= 1<<MASK_SPEAKING;
					}
					else{
						statusbyte_mask &= ~(1<<MASK_SPEAKING);
					}
					EEPROM_write(0x00, statusbyte_mask);							//save new settings to permanent memory
					LCDsendCommand(LCD_CLEAR);
					break;
				case 18 | (1<<MENU):												//if setting day of week
					statusbyte_menu = 1<<MENU;										//go to main menu
					TWIsendStart(DS3231_ADDR);										//send new day of week
					TWIsendByte(0x03);
					TWIsendByte(MenuDataArray[0]);
					TWIstop();
					if(statusbyte_mask & (1<<MASK_SPEAKING)){		//if speaking enabled
						WT588Dsay(WT_POTVRDENE);
						TWIsendStart(DS3231_ADDR);					//go to register address 0x00
						TWIsendByte(0x00);
						TWIstop();
						TWIreceive(DS3231_ADDR, RTCdata, 7);				//read 7 bytes (3 time bytes, 1 not used byte(day), 3 date bytes)
						dateValue[DAY_TENS] = (RTCdata[4] & 0x30)>>4;		//tens of day
						dateValue[DAY_ONES] = RTCdata[4] & 0x0F;			//ones of day
						dateValue[MONTH_TENS] = (RTCdata[5] & 0x10)>>4;		//tens of month
						dateValue[MONTH_ONES] = RTCdata[5] & 0x0F;			//ones of month
						dateValue[YEAR_TENS] = (RTCdata[6] & 0xF0)>>4;		//tens of years
						dateValue[YEAR_ONES] = RTCdata[6] & 0x0F;			//ones of years
						statusbyte_menu &= ~(1<<MENU);
						sayDate(dateValue);
						statusbyte_menu |= 1<<MENU;
					}
					break;
				case 19 | (1<<MENU):												//if setting volume
					statusbyte_menu = 1<<MENU;										//go to main menu
					EEPROM_write(0x01, volume);										//save value of volume to permanent memory
					WT588Dcommand(volume + 0xE0);									//send new volume to be set
			}
		}
		if(statusbyte_mask & (1<<MASK_SPEAKING)){			//if speaking enabled
			saySetting();
		}
	}
}

void statusTime(void){									//not menu, switch 1
	statusbyte_last_sw = ~(1<<SW1);						//set flag that this switch was already pressed - program will go trough this function although switch 1 won't be pressed
	if(!(statusbyte_sw & (1<<SW1))){					//if switch is pressed (for example if SW was pressed and is pressed again, then device should said/print data again)
		if(statusbyte_first & (1<<SW1)){				//if this is first program cycle in which program go trough this function
			statusbyte_first = ~(1<<SW1);				//clear flag of first time so next operations are done only once per switch press
			statusbyte_flags |= 1<<FLAGS_SAY;			//set flag of saying data
		}
	}
	else{												//if switch is not pressed
		statusbyte_first = 0xFF;						//set flag of first time
	}
}

void statusDate(void){									//not menu, switch 2
	statusbyte_last_sw = ~(1<<SW2);						//set flag that this switch was already pressed - program will go trough this function although switch 2 won't be pressed
	if(!(statusbyte_sw & (1<<SW2))){					//if switch is pressed (for example if SW was pressed and is pressed again, then device should said/print data again)
		if(statusbyte_first & (1<<SW2)){				//if this is first program cycle in which program go trough this function
			statusbyte_first = ~(1<<SW2);				//clear flag of first time so next operations are done only once per switch press
			statusbyte_mask |= 1<<MASK_DATE;			//set flag to print value on LCD
			statusbyte_flags |= 1<<FLAGS_SAY;			//set flag of saying data
		}
	}
	else{												//if switch is not pressed
		statusbyte_first = 0xFF;						//set flag of first time
	}
	
	if(statusbyte_mask & (1<<MASK_DATE)){				//if date should be print on LCD and said
		statusbyte_mask &= ~(1<<MASK_DATE);				//clear flag
		LCDclear4thLine();								//print date on LCD
		LCDputInt(dateValue[DAY_TENS]);
		LCDputInt(dateValue[DAY_ONES]);
		LCDwrite('.');
		LCDputInt(dateValue[MONTH_TENS]);
		LCDputInt(dateValue[MONTH_ONES]);
		LCDprintString(".20");
		LCDputInt(dateValue[YEAR_TENS]);
		LCDputInt(dateValue[YEAR_ONES]);
		LCDgoto(12, 3);
		switch(RTCdata[3]){									//print day of week
			case 1:
			LCDprintString("PONDELOK");
			break;
			case 2:
			LCDprintString("UTOROK");
			break;
			case 3:
			LCDprintString("STREDA");
			break;
			case 4:
			LCDprintString("STVRTOK");
			break;
			case 5:
			LCDprintString("PIATOK");
			break;
			case 6:
			LCDprintString("SOBOTA");
			break;
			case 7:
			LCDprintString("NEDELA");
			break;
		}
	}
}

void statusRH_T_in(void){								//not menu, switch 3
	statusbyte_last_sw = ~(1<<SW3);						//set flag that this switch was already pressed - program will go trough this function although switch 3 won't be pressed
	if(!(statusbyte_sw & (1<<SW3))){					//if switch is pressed (for example if SW was pressed and is pressed again, then device should said/print data again)
		if(statusbyte_first & (1<<SW3)){				//if this is first program cycle in which program go trough this function
			statusbyte_first = ~(1<<SW3);				//clear flag of first time so next operations are done only once per switch press
			statusbyte_mask |= 1<<MASK_RH_T_IN;			//set flag to print value on LCD
			statusbyte_flags |= 1<<FLAGS_SAY;			//set flag of saying data
		}
	}
	else{												//if switch is not pressed
		statusbyte_first = 0xFF;						//set flag of first time
	}
	if(statusbyte_mask & (1<<MASK_RH_T_IN)){			//if flag is set = press of switch or new data
		statusbyte_mask &= ~(1<<MASK_RH_T_IN);			//clear flag
		LCDclear4thLine();								//print temperature and humidity on LCD
		LCDgoto(2, 3);
		LCDputFloatOneDecPlace(temperatureIn);
		LCDwrite(0b11011111);
		LCDwrite('C');
		LCDgoto(13, 3);
		LCDputInt(humidityIn);
		LCDwrite('%');
	}
}

void statusRH_T_out(void){								//not menu, switch 4
	statusbyte_last_sw = ~(1<<SW4);						//set flag that this switch was already pressed - program will go trough this function although switch 4 won't be pressed
	if(!(statusbyte_sw & (1<<SW4))){					//if switch is pressed (for example if SW was pressed and is pressed again, then device should said/print data again)
		if(statusbyte_first & (1<<SW4)){				//if this is first program cycle in which program go trough this function
			statusbyte_first = ~(1<<SW4);				//clear flag of first time so next operations are done only once per switch press
			statusbyte_mask |= 1<<MASK_RH_T_OUT;		//set flag to print value on LCD
			statusbyte_flags |= 1<<FLAGS_SAY;			//set flag of saying data
		}
		if(resetTimer == mainTimer){					//if switch 4 is pressed long enough to reset
			resetTimer = 0;
			statusbyte_mask |= ((1<<MASK_SPEAKING) | (1<<MASK_LISTENING));
			volume = 4;
			EEPROM_write(0x00, statusbyte_mask & 0xB1);	//save settings
			EEPROM_write(0x01, volume);
			RVcommand(RV_WAIT);							//set RV module
			USART_Flush();								//clears FIFO USART register
			USARTreceived = 0xFF;
			char RVsetCounter = 5;
			while((USARTreceived == 0xFF) && (RVsetCounter > 0)){			//while didn't received feedback from RV module
				RVsend(0x24);							//send instruction which will return if sound instructions are recorded in RV module
				USART_Read();							//wait for feedback from RV module
				_delay_ms(30);
				RVsetCounter--;
			}
			if(USARTreceived == 0x01){					//if sound instructions are recorded
				RVcommand(RV_IMPORT_INSTRUCTIONS);		//start recognizing sound instructions
			}
			USARTreceived = NULL;						//clears variable for further purpose
			USART_Flush();								//clears FIFO USART register
			statusbyte_flags &= ~(1<<FLAGS_USART);
			WT588Dcommand(volume + 0xE0);
			WT588Dsay(WT_INTRO);
		}
	}
	else{												//if switch is not pressed
		statusbyte_first = 0xFF;						//set flag of first time
	}
	if(statusbyte_mask & (1<<MASK_RH_T_OUT)){			//if flag is set = press of switch or new data
		statusbyte_mask &= ~(1<<MASK_RH_T_OUT);			//clear flag
		statusbyte_flags &= ~(1<<FLAGS_RF_NOT_SAME);
		for(char i = 0; i < 6; i++){
			if(RFreceivedData[0][i] != RFreceivedData[1][i]){
				statusbyte_flags |= 1<<FLAGS_RF_NOT_SAME;
			}
		}
		if(RFreceivedData[0][0] > 0x0F){
			statusbyte_flags |= 1<<FLAGS_RF_NOT_SAME;
		}
		if(!(statusbyte_flags & (1<<FLAGS_RF_NOT_SAME))){
			for(char i = 0; i < 6; i++){
				outdoorSensor[i] = RFreceivedData[0][i];
			}
		}
		
		statusbyte_flags &= ~(1<<FLAGS_RF_NOT_SAME);
		for(char i = 0; i < 6; i++){
			if(RFreceivedData[1][i] != RFreceivedData[2][i]){
				statusbyte_flags |= 1<<FLAGS_RF_NOT_SAME;
			}
		}
		if(RFreceivedData[1][0] > 0x0F){
			statusbyte_flags |= 1<<FLAGS_RF_NOT_SAME;
		}
		if(!(statusbyte_flags & (1<<FLAGS_RF_NOT_SAME))){
			for(char i = 0; i < 6; i++){
				outdoorSensor[i] = RFreceivedData[1][i];
			}
		}
		
		statusbyte_flags &= ~(1<<FLAGS_RF_NOT_SAME);
		for(char i = 0; i < 6; i++){
			if(RFreceivedData[2][i] != RFreceivedData[0][i]){
				statusbyte_flags |= 1<<FLAGS_RF_NOT_SAME;
			}
		}
		if(RFreceivedData[2][0] > 0x0F){
			statusbyte_flags |= 1<<FLAGS_RF_NOT_SAME;
		}
		if(!(statusbyte_flags & (1<<FLAGS_RF_NOT_SAME))){
			for(char i = 0; i < 6; i++){
				outdoorSensor[i] = RFreceivedData[0][i];
			}
		}
		statusbyte_flags &= ~(1<<FLAGS_RF_NOT_SAME);
		if(outdoorSensor[SENSOR_STATUS] & (1<<SENSOR_STATUS_LOW_BATTERY)){
			LCDclear4thLine();
			LCDgoto(1, 3);
			LCDprintString("NIZKY STAV BATERIE");
			_delay_ms(5000);
		}
		LCDclear4thLine();								//print temperature and humidity on LCD
		LCDgoto(2, 3);
		if(outdoorSensor[SENSOR_STATUS] & (1<<SENSOR_STATUS_MINUS)){
			LCDwrite('-');
		}
		if(outdoorSensor[SENSOR_T_TENS]){
			LCDwrite(outdoorSensor[SENSOR_T_TENS] + '0');
		}
		LCDwrite(outdoorSensor[SENSOR_T_ONES] + '0');
		LCDwrite('.');
		LCDwrite(outdoorSensor[SENSOR_T_DECIMAL] + '0');
		LCDwrite(0b11011111);
		LCDwrite('C');
		LCDgoto(13, 3);
		LCDwrite(outdoorSensor[SENSOR_H_TENS] + '0');
		LCDwrite(outdoorSensor[SENSOR_H_ONES] + '0');
		LCDwrite('%');
	}
}

void statusMenuIn(void){								//not menu switch 5
	statusbyte_last_sw = ~(1<<SW5);						//set flag that this switch was already pressed - program will go trough this function although switch 2 won't be pressed
	if(!(statusbyte_sw & (1<<SW5))){					//if switch is pressed
		if(switchesTimer == mainTimer){					//if switch 5 is pressed long enough to go to menu
			switchesTimer = 0;
			if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
				WT588Dsay(WT_MENU_IN);					//say "vstup do menu"
			}
			LCDsendCommand(LCD_CLEAR);					//print go to menu information on LCD
			LCDgoto(8, 1);
			LCDprintString("MENU");
			statusbyte_flags |= 1<<FLAGS_GO_TO_MENU;	//set flag of go to menu, so program will continuous to menu after switch won't be pressed
			if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
				WT588Dsay(WT_MENU_POPIS);				//say description of menu
			}
			_delay_ms(2000);
		}
	}
	else{												//if switch 5 is not pressed
		if(statusbyte_flags & (1<<FLAGS_GO_TO_MENU)){	//it was long press
			statusbyte_flags &= ~(1<<FLAGS_GO_TO_MENU);	//clear flag go to menu
			statusbyte_menu |= 1<<MENU;					//set flag of menu
			statusbyte_sw = 0xFF;						//clear and set flags as none of switches is and was pressed
			statusbyte_last_sw = 0xFF;
			statusbyte_first = 0xFF;
			statusbyte_flags |= 1<<FLAGS_MENU_LCD;		//set flag of LCD update
		}
		else{											//short press
			LCDsendCommand(LCD_CLEAR);					//print information of alarm enable/disable change
			LCDgoto(7, 1);
			LCDprintString("BUDIK:");
			LCDgoto(7, 2);
			if(statusbyte_mask & (1<<MASK_ALARM)){		//change state of alarm enable
				statusbyte_mask &= ~(1<<MASK_ALARM);
				LCDprintString("VYPNUTY");					//print if alarm is enabled or disabled
				if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
					WT588Dsay(WT_BUDIK_VYPNUTY);			//say alarm disabled
				}
			}
			else{
				statusbyte_mask |= 1<<MASK_ALARM;
				LCDprintString("ZAPNUTY");
				TWIsendStart(DS3231_ADDR);					//clear alarm flag in DS3231
				TWIsendByte(0x0F);
				TWIsendByte(0x00);
				TWIstop();
				if(statusbyte_mask & (1<<MASK_SPEAKING)){	//if speaking enabled
					WT588Dsay(WT_BUDIK_ZAPNUTY);			//say alarm enabled
				}
			}
			EEPROM_write(0x00, statusbyte_mask);		//save new settings to permanent memory
			_delay_ms(2000);
			statusMenuOut();							//go back to main program
		}
	}
}

void statusMenuOut(void){								//used to go back to main program
	if(statusbyte_menu & (1<<MENU)){
		statusbyte_menu &= ~(1<<MENU);					//clear flag of menu
		if(statusbyte_mask & (1<<MASK_SPEAKING)){		//if speaking enabled
			WT588Dcommand(WT588D_STOP);
			WT588Dsay(WT_MENU_OUT);						//say "vstup z menu"
		}
	}
	while(SHR(SHR_SWITCHES) != 0xFF){					//wait until all switches aren't pushed
		;
	}
	statusbyte_sw = 0xFF;								//clear flags = all switches aren't pushed
	statusbyte_last_sw = 0xFF;
	statusbyte_first = 0xFF;
	LCDsendCommand(LCD_CLEAR);
}

void SetVoiceRecognizing(void){							//recording voice instructions to RV module
	LCDsendCommand(LCD_CLEAR);
	LCDgoto(6, 1);
	LCDprintString("pockajte");
	LCDgoto(7, 2);
	LCDprintString("prosim");
	RVcommand(RV_WAIT);									//send wait command
	RVcommand(RV_DELETE_INSTRUCTIONS);					//delete old voice instructions
	RVcommand(RV_WAIT);									//send wait command
	USART_Flush();
	while(USARTreceived != 0x40){						//while received data is not confirmation of recording voice instructions
		RVsend(0x11);									//send recording voice instructions command
		USART_Read();									//wait for feedback from RV module
		_delay_ms(30);
	}
	USARTreceived = NULL;
	_delay_ms(30);
	LCDprintString("PRIKAZ:");							//print information on LCD
	LCDgoto(0, 1);
	LCDprintString("povedz cas");
	LCDgoto(0, 3);
	LCDprintString("HOVORTE PRIKAZ");
	while(USARTreceived != 0x46){						//while received feedback from RV module is not successfully recorded all voice instructions
		LCDgoto(0, 0);
		LCDprintString("PRIKAZ:");
		LCDgoto(0, 1);
		switch(RVsettingStatus){						//print which voice instruction should be recorded right now
			case 0:
				LCDprintString("povedz cas          ");
				break;
			case 1:
				LCDprintString("povedz datum        ");
				break;
			case 2:
				LCDprintString("povedz teplotu dnu  ");
				break;
			case 3:
				LCDprintString("povedz teplotu vonku");
				break;
			case 4:
				LCDprintString("zmen budik          ");
				break;
		}
		LCDgoto(0, 2);
		LCDprintString("                    ");
		LCDgoto(0, 3);
		switch(USARTreceived){							//print what should man do based on RV module data
			case 0x40:
				LCDprintString("HOVORTE PRIKAZ      ");
				break;
			case 0x41:
				LCDprintString("ZIADEN ZVUK         ");
				break;
			case 0x42:
				LCDprintString("CAKAJTE A ZOPAKUJTE ");
				break;
			case 0x43:
				LCDprintString("PRILIS NAHLAS       ");
				break;
			case 0x44:
				LCDprintString("SA NEZHODUJE        ");
				LCDgoto(0, 2);
				LCDprintString("ZOPAKOVANY PRIKAZ   ");
				break;
			case 0x45:
				LCDprintString("PRIKAZ ZAZNAMENANY  ");
				RVsettingStatus++;
				break;
		}
		while(!(UCSR0A & (1<<RXC0))){	//wait for feedback from RV module
			;
		}
		USARTreceived = UDR0;
	}
	USARTreceived = NULL;
	RVcommand(RV_WAIT);									//send wait command
	RVcommand(RV_IMPORT_INSTRUCTIONS);					//send command for recognizing voice instructions
	LCDsendCommand(LCD_CLEAR);
	LCDgoto(5, 1);										//print information of successfully recorded voice instructions
	LCDprintString("PRIKAZY SU");
	LCDgoto(2, 2);
	LCDprintString("USPESNE NAHRATE");
	_delay_ms(3000);
	statusbyte_menu = 1<<MENU;							//set flag to go to main menu
	statusbyte_flags |= (1<<FLAGS_MENU_LCD);			//set flag of LCD update
}

void speaking(void){									//say required information
	if(statusbyte_mask & (1<<MASK_SPEAKING)){			//if flag of saying is set
		if(statusbyte_flags & (1<<FLAGS_SAY)){			//if should by said some information
			statusbyte_flags &= ~(1<<FLAGS_SAY);		//clear flag
			statusbyte_flags &= ~(1<<FLAGS_USART);		//clear flag - data received from RV was said
			float temperatureOut;
			float humidityOut;
			switch(statusbyte_state | 0xF0){			//depends on state
				case 0b11111110:						//time
					sayTime(timeValue);
					break;
				case 0b11111101:						//date ...
					sayDate(dateValue);
					break;
				case 0b11111011:
					WT588Dsay(WT_VO_VNUTRI_JE);			//say "vo vntri je"
					sayRH_T(temperatureIn, humidityIn);	//say values
					break;
				case 0b11110111:
					temperatureOut = (outdoorSensor[SENSOR_T_TENS] * 10) + outdoorSensor[SENSOR_T_ONES];
					temperatureOut += (float)outdoorSensor[SENSOR_T_DECIMAL] / 10;
					if(outdoorSensor[SENSOR_STATUS] & (1<<SENSOR_STATUS_MINUS)){
						temperatureOut *= -1;
					}
					humidityOut = (outdoorSensor[SENSOR_H_TENS] * 10) + outdoorSensor[SENSOR_H_ONES];
					if(outdoorSensor[SENSOR_STATUS] & (1<<SENSOR_STATUS_LOW_BATTERY)){
						WT588Dsay(WT_NIZKA_BATERIA);		//say low battery
						_delay_ms(1000);
					}
					WT588Dsay(WT_VONKU_JE);					//say "vonku je"
					sayRH_T(temperatureOut, humidityOut);
					break;
			}
			WT588Dsay(0x80);
			USART_Flush();
		}
	}
	else{												//if is speaking disabled
		if(statusbyte_flags & (1<<FLAGS_SAY)){
			statusbyte_flags &= ~(1<<FLAGS_USART);		//clear flag - data received from RV would be said if speaking was enabled
		}
		statusbyte_flags &= ~(1<<FLAGS_SAY);			//clear flag of saying some information
	}
}

void sayTime(char *sayTimeBuffer){
	char sayTimeHours = (sayTimeBuffer[HOURS_TENS] * 10) + sayTimeBuffer[HOURS_ONES];		//value of day
	char sayTimeMinutes = (sayTimeBuffer[MINUTES_TENS] * 10) + sayTimeBuffer[MINUTES_ONES];	//value of day
	if(! statusbyte_menu){												//if not menu
		if((sayTimeHours > 4) || (sayTimeHours < 2)){					//if hours are more than 4 or less than two
			WT588Dsay(WT_JE);											//say "je"
		}
		else{
			WT588Dsay(WT_SU);											//say "s"
		}
	}
	if((!(statusbyte_menu & (1<<MENU))) || (((statusbyte_menu == 0x83) || (statusbyte_menu == 0x84)) || ((statusbyte_menu == 0x8D) || (statusbyte_menu == 0x8E)))){
		if(sayTimeBuffer[HOURS_TENS]){									//if tens of hours isn't 0
			if(sayTimeBuffer[HOURS_TENS] == 1){
				WT588Dsay(sayTimeBuffer[HOURS_ONES] + WT_10);			//numbers 10-19 
			}
			if(sayTimeBuffer[HOURS_TENS] == 2){
				WT588Dsay(WT_20);										//20
			}
		}
		if(sayTimeHours == 1){											//if one hour
			WT588Dsay(WT_1);											//say "jedna"
		}
		else if(sayTimeHours == 2){										//if two hours
			WT588Dsay(WT_2);											//say "dve"
		}
		else if((sayTimeBuffer[HOURS_ONES] || (sayTimeBuffer[HOURS_TENS] == 0)) && (sayTimeBuffer[HOURS_TENS] != 1)){	//if hours are 0, ones of hours isn't zero or hours aren't 10-19
			WT588Dsay(sayTimeBuffer[HOURS_ONES]);						//say ones of hours
		}
	}
	if(!(statusbyte_menu & (1<<MENU))){
		if(sayTimeHours == 1){											//if one hour
			WT588Dsay(WT_HODINA);										//say "hodina"
		}
		else if((sayTimeBuffer[HOURS_TENS] == 0) && ((sayTimeBuffer[HOURS_ONES] > 1) && (sayTimeBuffer[HOURS_ONES] < 5))){	//if more than 1 hour but less than 5 hours
			WT588Dsay(WT_HODINY);										//say "hodiny"
		}
		else{															//if more than 4 hours
			WT588Dsay(WT_HODIN);										//say "hodn"
		}
	}
/*----------------------------------------------------------------------------------------------*/
	if((!(statusbyte_menu & (1<<MENU))) || (((statusbyte_menu == 0x81) || (statusbyte_menu == 0x82)) || ((statusbyte_menu == 0x8B) || (statusbyte_menu == 0x8C)))){
		if(sayTimeBuffer[MINUTES_TENS]){								//if tens of minutes isn't 0
			if(sayTimeBuffer[MINUTES_TENS] == 1){
				WT588Dsay(sayTimeBuffer[MINUTES_ONES] + WT_10);			//numbers 10-19
			}
			else{
				WT588Dsay(sayTimeBuffer[MINUTES_TENS] + WT_18);			//numbers 20, 30, 40, 50
			}
		}
		if(sayTimeMinutes == 1){										//if one minute
			WT588Dsay(WT_1);											//say "jedna"
		}
		else if(sayTimeMinutes == 2){									//if two minutes
			WT588Dsay(WT_2);											//say "dve"
		}
		else if((sayTimeBuffer[MINUTES_ONES] || (sayTimeBuffer[MINUTES_TENS] == 0)) && (sayTimeBuffer[MINUTES_TENS] != 1)){
			WT588Dsay(sayTimeBuffer[MINUTES_ONES]);
		}
	}
	if(!(statusbyte_menu & (1<<MENU))){
		if((sayTimeBuffer[MINUTES_TENS] == 0) && (sayTimeBuffer[MINUTES_ONES] == 1)){	//if one minute
			WT588Dsay(WT_MINUTA);										//say "minta"
		}
		else if((sayTimeBuffer[MINUTES_TENS] == 0) && ((sayTimeBuffer[MINUTES_ONES] > 1) && (sayTimeBuffer[MINUTES_ONES] < 5))){	//if more than 1 hour but less than 5 minutes
			WT588Dsay(WT_MINUTY);										//say "minty"
		}
		else{															//if more than 4 minutes
			WT588Dsay(WT_MINUT);										//say "mint"
		}
	}
}

void sayDate(char *sayDateBuffer){										//say day of week and date
	if(!(statusbyte_menu & (1<<MENU))){
		WT588Dsay(RTCdata[3] + (WT_PONDELOK - 1));
		_delay_ms(250);													//space between day of week and date
	}
	if((!(statusbyte_menu & (1<<MENU))) || ((statusbyte_menu == 0x85) || (statusbyte_menu == 0x86))){
		if(sayDateBuffer[DAY_TENS]){									//if tens of days isn't 0
			if(sayDateBuffer[DAY_TENS] == 1){
				WT588Dsay((sayDateBuffer[DAY_ONES]) + WT_10th);			//numbers 10th-19th
			}
			else{
				WT588Dsay(sayDateBuffer[DAY_TENS] + WT_18th);			//numbers 20th, 30th, 40th, 50th
			}
		}
		if((sayDateBuffer[DAY_ONES]) && (sayDateBuffer[DAY_TENS] != 1)){	//if ones of day is not 0 and tens of days are not 10-19
			WT588Dsay(sayDateBuffer[DAY_ONES] + WT_1st_MINUS_1);		//say ones of day
		}
	}
	if((!(statusbyte_menu & (1<<MENU))) || (statusbyte_menu == 0x87)){
		if(!(statusbyte_menu & (1<<MENU))){
			WT588Dsay(((sayDateBuffer[MONTH_TENS] * 10) + sayDateBuffer[MONTH_ONES]) + WT_JANUAR_MINUS_1);	//say month
		}
		else{
			WT588Dsay(sayDateBuffer[MONTH_TENS] + WT_JANUAR_MINUS_1);
		}
	}
	if((!(statusbyte_menu & (1<<MENU))) || ((statusbyte_menu == 0x89) || (statusbyte_menu == 0x8A))){
		WT588Dsay(WT_2000);												//say "two thousands"
		if(sayDateBuffer[YEAR_TENS]){									//if tens of minutes isn't 0
			if(sayDateBuffer[YEAR_TENS] == 1){
				WT588Dsay((sayDateBuffer[YEAR_ONES]) + WT_10);			//numbers 10th-19th
			}
			else{
				WT588Dsay(sayDateBuffer[YEAR_TENS] + WT_18);			//numbers 20th, 30th, 40th, 50th
			}
		}
		if((sayDateBuffer[YEAR_ONES]) && (sayDateBuffer[YEAR_TENS] != 1)){	//if ones of year is not 0 and tens of year are not 10-19
			WT588Dsay(sayDateBuffer[YEAR_ONES]);						//say ones of year
		}
	}
}

void sayRH_T(float sayRH_T_temp, char sayRH_T_hum){	//say temperature and humidity
	int ones, tens, decimal;
	if(sayRH_T_temp < 0){							//if minus
		sayRH_T_temp *= -1;							//convert to plus
		WT588Dsay(WT_MINUS);						//say "mnus"
	}
	ones = sayRH_T_temp;
	tens = ones / 10;
	decimal = sayRH_T_temp * 10;
	decimal -= ones * 10;
	ones -= tens * 10;
	if(tens){										//if tens isn't 0
		if(tens == 1){
			WT588Dsay(ones + WT_10);				//numbers 10-19
		}
		else{
			WT588Dsay(tens + WT_18);				//numbers 20, 30, 40, 50
		}
	}
	if((ones || (tens == 0)) && (tens != 1)){		//if number 0, ones isn't zero or number isn't 10-19
		WT588Dsay(ones);							//say ones
	}
	WT588Dsay(WT_CELA);								//say "cel"
	WT588Dsay(decimal);								//say number after decimal point
	WT588Dsay(WT_STUPNOV);							//say "stupov celzia a vlhkos"
	tens = sayRH_T_hum / 10;
	ones = sayRH_T_hum - (tens * 10);
	if(tens){										//if tens isn't 0
		if(tens == 1){
			WT588Dsay(ones + WT_10);				//numbers 10-19
		}
		else{
			WT588Dsay(tens + WT_18);				//numbers 20, 30, 40, 50
		}
	}
	if((ones || (tens == 0)) && (tens != 1)){		//if number 0, ones isn't zero or number isn't 10-19
		WT588Dsay(ones);							//say ones
	}
	WT588Dsay(WT_PERCENT);							//say "percent"
}

void saySetting(void){
	if(statusbyte_menu != 0x91){
		switch(statusbyte_menu){
			case 1<<MENU:
				WT588Dsay(WT_POTVRDENE);
				WT588Dsay(WT_MENU_IN);
				WT588Dsay(WT_MENU_POPIS);
				break;
			case 1 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_CAS);
				_delay_ms(1000);
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_JEDNOTKY);
				WT588Dsay(WT_MINUT);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 2 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_DESIATKY);
				WT588Dsay(WT_MINUT);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 3 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_JEDNOTKY);
				WT588Dsay(WT_HODIN);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 4 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_DESIATKY);
				WT588Dsay(WT_HODIN);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 5 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_DATUM);
				_delay_ms(1500);
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_DESIATKY);
				WT588Dsay(WT_DNA);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 6 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_JEDNOTKY);
				WT588Dsay(WT_DNA);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 7 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_MESIAC);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 9 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_DESIATKY);
				WT588Dsay(WT_ROKOV);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 10 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_JEDNOTKY);
				WT588Dsay(WT_ROKOV);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 18 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_DEN_V_TYZDNI);
				break;
			case 11 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_CAS_BUDIKA);
				_delay_ms(1500);
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_JEDNOTKY);
				WT588Dsay(WT_MINUT);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 12 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_DESIATKY);
				WT588Dsay(WT_MINUT);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 13 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_JEDNOTKY);
				WT588Dsay(WT_HODIN);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 14 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_DESIATKY);
				WT588Dsay(WT_HODIN);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 15 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_HLASOVY_VYSTUP);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 16 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_ROZPOZNAVANIE_HLASU);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
			case 17 | (1<<MENU):
				WT588Dsay(WT_NASTAVTE);
				WT588Dsay(WT_HLASITOST);
				WT588Dsay(WT_POTVRTDE_TLACIDLOM);
				break;
		}
	}
}